from pathlib import PurePath
from tests.integration_tests.validation.conformance_suite_config import ConformanceSuiteConfig

TIMING = {f'tests/inline_xbrl/{k}': v for k, v in {
    'G2-1-2/index.xml': 8.701,
    'G2-1-3_1/index.xml': 4.427,
    'G2-1-3_2/index.xml': 4.459,
    'G2-2-1/index.xml': 4.301,
    'G2-2-2/index.xml': 4.305,
    'G2-2-3/index.xml': 13.038,
    'G2-2-6/index.xml': 6.555,
    'G2-2-7/index.xml': 8.803,
    'G2-3-1_2/index.xml': 10.617,
    'G2-4-1_1/index.xml': 8.830,
    'G2-4-1_2/index.xml': 4.321,
    'G2-4-1_3/index.xml': 4.347,
    'G2-4-2_1/index.xml': 4.364,
    'G2-4-2_2/index.xml': 4.323,
    'G2-5-1_1/index.xml': 8.591,
    'G2-5-1_2/index.xml': 8.633,
    'G2-5-1_3/index.xml': 6.587,
    'G2-5-2/index.xml': 8.620,
    'G2-5-3/index.xml': 4.337,
    'G2-5-4_1/index.xml': 4.275,
    'G2-5-4_2/index.xml': 4.382,
    'G2-5-4_3/index.xml': 4.228,
    'G2-6-1_1/index.xml': 6.481,
    'G2-6-1_2/index.xml': 6.711,
    'G2-6-1_3/index.xml': 2.446,
    'G2-6-2/index.xml': 4.579,
    'G2-7-1_1/index.xml': 4.459,
    'G2-7-1_2/index.xml': 4.668,
    'G3-1-1_1/index.xml': 16.013,
    'G3-1-1_2/index.xml': 10.167,
    'G3-1-2/index.xml': 16.833,
    'G3-1-5/index.xml': 17.252,
    'G3-2-2/index.xml': 4.282,
    'G3-2-3/index.xml': 4.313,
    'G3-4-2_1/index.xml': 4.376,
    'G3-4-2_2/index.xml': 4.401,
    'G3-4-2_3/index.xml': 4.303,
    'G3-4-2_4/index.xml': 6.477,
    'G3-4-3_1/index.xml': 6.529,
    'G3-4-3_2/index.xml': 6.576,
    'G3-4-4/index.xml': 4.393,
    'G3-4-5_1/index.xml': 4.320,
    'G3-4-5_2/index.xml': 6.512,
    'G3-4-6/index.xml': 6.446,
    'G3-4-7/index.xml': 4.410,
    'G3-5-1/index.xml': 4.391,
    'RTS_Annex_III_Par_1/index.xml': 6.577,
    'RTS_Annex_III_Par_3_G3-1-3/index.xml': 2.200,
    'RTS_Annex_II_Par_1/index.xml': 6.552,
    'RTS_Annex_II_Par_1_RTS_Annex_IV_par_7/index.xml': 4.323,
    'RTS_Annex_IV_Par_11_G3-2-2/index.xml': 6.551,
    'RTS_Annex_IV_Par_12_G2-2-4/index.xml': 15.244,
    'RTS_Annex_IV_Par_14_G2-5-1/index.xml': 4.316,
    'RTS_Annex_IV_Par_1_G2-1-4/index.xml': 4.334,
    'RTS_Annex_IV_Par_2_G2-1-1/index.xml': 6.475,
    'RTS_Annex_IV_Par_4_1/index.xml': 4.316,
    'RTS_Annex_IV_Par_4_2/index.xml': 4.360,
    'RTS_Annex_IV_Par_4_G1-1-1_G3-4-5/index.xml': 11.678,
    'RTS_Annex_IV_Par_5/index.xml': 6.588,
    'RTS_Annex_IV_Par_6/index.xml': 4.328,
    'RTS_Annex_IV_Par_8_G3-4-5/index.xml': 6.459,
    'RTS_Annex_IV_Par_9_Par_10_G1-4-1_G1-4-2_G3-3-1_G3-3-2/index.xml': 17.520,
    'RTS_Art_3/index.xml': 8.643,
    'RTS_Art_6_a/index.xml': 4.175,
}.items()}

config = ConformanceSuiteConfig(
    approximate_relative_timing=TIMING,
    args=[
        '--disclosureSystem', 'esef-2023',
        '--formula', 'run',
    ],
    expected_failure_ids=frozenset(f'tests/inline_xbrl/{s}' for s in [
        # Test report uses older domain item type (http://www.xbrl.org/dtr/type/non-numeric) forbidden by ESEF.3.2.2.
        'G3-1-2/index.xml:TC2_valid',
        # These tests reference zip files, which do not exist in the conformance suite.
        'G2-6-1_3/index.xml:TC2_invalid',
        'G2-6-1_3/index.xml:TC3_invalid',
        # The following test case fails because of the `tech_duplicated_facts1` formula, which incorrectly fires.
        # It does not take into account the language attribute on the fact.
        # Facts are not duplicates if their language attributes are different.
        'RTS_Annex_IV_Par_12_G2-2-4/index.xml:TC5_valid',
    ]),
    file='index_inline_xbrl.xml',
    info_url='https://www.esma.europa.eu/document/esef-conformance-suite-2023',
    local_filepath='esef_conformance_suite_2023.zip',
    name=PurePath(__file__).stem,
    plugins=frozenset({'validate/ESEF'}),
    public_download_url='https://www.esma.europa.eu/sites/default/files/2023-12/esef_conformance_suite_2023.zip',
    shards=8,
)



from pathlib import PurePath
from tests.integration_tests.validation.conformance_suite_config import ConformanceSuiteConfig

TIMING = {f'esef_conformance_suite_2021/esef_conformance_suite_2021/tests/inline_xbrl/{k}': v for k, v in {
    'G2-1-2/index.xml': 8.724,
    'G2-1-3_1/index.xml': 4.393,
    'G2-1-3_2/index.xml': 4.452,
    'G2-2-1/index.xml': 4.358,
    'G2-2-2/index.xml': 4.567,
    'G2-2-3/index.xml': 11.065,
    'G2-3-1_1/index.xml': 8.674,
    'G2-3-1_2/index.xml': 4.588,
    'G2-3-1_3/index.xml': 11.361,
    'G2-4-1_1/index.xml': 9.050,
    'G2-4-1_2/index.xml': 4.471,
    'G2-4-1_3/index.xml': 4.736,
    'G2-4-2_1/index.xml': 4.304,
    'G2-4-2_2/index.xml': 4.405,
    'G2-5-1_1/index.xml': 8.771,
    'G2-5-1_2/index.xml': 8.736,
    'G2-5-1_3/index.xml': 6.685,
    'G2-5-2/index.xml': 8.961,
    'G2-5-3/index.xml': 4.413,
    'G2-5-4_1/index.xml': 4.287,
    'G2-5-4_2/index.xml': 4.457,
    'G2-5-4_3/index.xml': 4.415,
    'G2-6-1_1/index.xml': 6.262,
    'G2-6-1_2/index.xml': 6.405,
    'G2-6-2/index.xml': 4.485,
    'G2-7-1_1/index.xml': 4.094,
    'G2-7-1_2/index.xml': 4.231,
    'G3-1-1_1/index.xml': 14.967,
    'G3-1-1_2/index.xml': 9.589,
    'G3-1-2/index.xml': 13.141,
    'G3-1-5/index.xml': 16.767,
    'G3-2-1/index.xml': 4.158,
    'G3-2-2/index.xml': 4.156,
    'G3-2-3/index.xml': 4.181,
    'G3-2-5/index.xml': 4.155,
    'G3-4-2_1/index.xml': 4.137,
    'G3-4-2_2/index.xml': 4.163,
    'G3-4-2_3/index.xml': 4.138,
    'G3-4-2_4/index.xml': 6.291,
    'G3-4-3_1/index.xml': 6.240,
    'G3-4-3_2/index.xml': 6.304,
    'G3-4-4/index.xml': 4.362,
    'G3-4-5_1/index.xml': 4.318,
    'G3-4-5_2/index.xml': 4.320,
    'G3-4-7/index.xml': 4.248,
    'G3-5-1/index.xml': 4.179,
    'RTS_Annex_III_Par_1/index.xml': 6.262,
    'RTS_Annex_III_Par_3_G3-1-3/index.xml': 2.108,
    'RTS_Annex_II_Par_1/index.xml': 6.297,
    'RTS_Annex_II_Par_1_RTS_Annex_IV_par_7/index.xml': 4.193,
    'RTS_Annex_II_Par_2/index.xml': 4.268,
    'RTS_Annex_IV_Par_11_G3-2-2/index.xml': 6.295,
    'RTS_Annex_IV_Par_12_G2-2-4/index.xml': 8.431,
    'RTS_Annex_IV_Par_14_G2-5-1/index.xml': 4.183,
    'RTS_Annex_IV_Par_1_G2-1-4/index.xml': 4.157,
    'RTS_Annex_IV_Par_2_G2-1-1/index.xml': 6.318,
    'RTS_Annex_IV_Par_4_1/index.xml': 4.206,
    'RTS_Annex_IV_Par_4_2/index.xml': 4.202,
    'RTS_Annex_IV_Par_4_G1-1-1_G3-4-5/index.xml': 11.126,
    'RTS_Annex_IV_Par_5_G3-4-6/index.xml': 10.570,
    'RTS_Annex_IV_Par_6/index.xml': 4.184,
    'RTS_Annex_IV_Par_8/index.xml': 6.279,
    'RTS_Annex_IV_Par_9_Par_10_G1-4-1_G1-4-2_G3-3-1_G3-3-2/index.xml': 16.773,
    'RTS_Art_3/index.xml': 8.407,
    'RTS_Art_6_a/index.xml': 4.066,
}.items()}

config = ConformanceSuiteConfig(
    approximate_relative_timing=TIMING,
    args=[
        '--disclosureSystem', 'esef-2021',
        '--formula', 'run',
    ],
    file='esef_conformance_suite_2021/esef_conformance_suite_2021/index_inline_xbrl.xml',
    info_url='https://www.esma.europa.eu/document/conformance-suite-2021',
    local_filepath='esef_conformance_suite_2021.zip',
    name=PurePath(__file__).stem,
    plugins=frozenset({'validate/ESEF'}),
    public_download_url='https://www.esma.europa.eu/sites/default/files/library/esef_conformance_suite_2021.zip',
    shards=8,
)



from pathlib import PurePath
from tests.integration_tests.validation.conformance_suite_config import ConformanceSuiteConfig

TIMING = {
    'fg_nl/04-testcase.xml': 0.019,
    'fg_nl/05-testcase.xml': 0.012,
    'fg_nl/09-testcase.xml': 0.011,
    'fg_nl/11-testcase.xml': 0.011,
    'fr_nl/1-01-testcase.xml': 0.002,
    'fr_nl/1-02-testcase.xml': 0.002,
    'fr_nl/1-03-testcase.xml': 0.001,
    'fr_nl/1-04-testcase.xml': 0.002,
    'fr_nl/1-05-testcase.xml': 0.001,
    'fr_nl/1-06-testcase.xml': 0.001,
    'fr_nl/2-03-testcase.xml': 0.001,
    'fr_nl/2-06-testcase.xml': 0.002,
    'fr_nl/3-04-testcase.xml': 2.350,
    'fr_nl/4-01-testcase.xml': 1.610,
    'fr_nl/5-01-testcase.xml': 1.594,
    'fr_nl/5-03-testcase.xml': 0.806,
    'fr_nl/5-11-testcase.xml': 0.025,
    'fr_nl/6-01-testcase.xml': 0.794,
}

config = ConformanceSuiteConfig(
    approximate_relative_timing=TIMING,
    args=[
        '--disclosureSystem', 'NT18-preview',
    ],
    file='index.xml',
    info_url='https://sbr-nl.nl/sites/default/files/bestanden/taxonomie/SBR%20Filing%20Rules%20NT18%20-%2020230301_.pdf',
    local_filepath='nl_nt18',
    name=PurePath(__file__).stem,
    plugins=frozenset({'validate/NL'}),
    shards=4,
)



import os
from pathlib import PurePath
from tests.integration_tests.validation.conformance_suite_config import (
    CONFORMANCE_SUITE_PATH_PREFIX, ConformanceSuiteConfig
)

TIMING = {f'inlineXBRL-1.1-conformanceSuite-2020-04-08/tests/{k}': v for k, v in {
    'baseURIs/FAIL-baseURI-on-ix-header.xml': 0.058,
    'baseURIs/FAIL-baseURI-on-xhtml.xml': 0.046,
    'baseURIs/PASS-baseURI-on-ix-references-multiRefs.xml': 0.075,
    'continuation/FAIL-continuation-duplicate-id.xml': 0.054,
    'continuation/FAIL-continuation-nonNumeric-circular.xml': 0.053,
    'continuation/FAIL-continuation-nonNumeric-circular2.xml': 0.056,
    'continuation/FAIL-continuation-nonNumeric-invalid-nesting-2.xml': 0.055,
    'continuation/FAIL-continuation-nonNumeric-invalid-nesting.xml': 0.053,
    'continuation/FAIL-continuation-nonNumeric-self.xml': 0.054,
    'continuation/FAIL-continuation-orphaned-cycle.xml': 0.054,
    'continuation/FAIL-continuation-used-twice.xml': 0.052,
    'continuation/FAIL-footnote-continuation-invalid-nesting-2.xml': 0.054,
    'continuation/FAIL-footnote-continuation-invalid-nesting.xml': 0.053,
    'continuation/FAIL-nonNumeric-dangling-continuation-2.xml': 0.053,
    'continuation/FAIL-nonNumeric-dangling-continuation.xml': 0.056,
    'continuation/FAIL-orphaned-continuation.xml': 0.045,
    'continuation/PASS-nonNumeric-continuation-multiple-documents.xml': 0.096,
    'continuation/PASS-nonNumeric-continuation-other-descendants-escaped.xml': 0.063,
    'continuation/PASS-nonNumeric-continuation-other-descendants.xml': 0.061,
    'continuation/PASS-nonNumeric-continuation-out-of-order.xml': 0.063,
    'continuation/PASS-nonNumeric-continuation-transform.xml': 0.059,
    'continuation/PASS-nonNumeric-continuation.xml': 0.077,
    'exclude/FAIL-exclude-nonFraction-parent.xml': 0.057,
    'exclude/FAIL-misplaced-exclude.xml': 0.045,
    'exclude/PASS-element-ix-exclude-complete.xml': 1.484,
    'exclude/PASS-exclude-nonNumeric-parent.xml': 0.103,
    'exclude/PASS-multiple-excludes-nonNumeric-parent.xml': 0.064,
    'footnotes/FAIL-element-ix-footnote-04.xml': 1.314,
    'footnotes/FAIL-footnote-any-attribute.xml': 0.057,
    'footnotes/FAIL-footnote-dangling-continuation.xml': 0.048,
    'footnotes/FAIL-footnote-dangling-fromRef.xml': 0.055,
    'footnotes/FAIL-footnote-dangling-toRef.xml': 0.053,
    'footnotes/FAIL-footnote-duplicate-footnoteIDs-different-input-docs.xml': 0.096,
    'footnotes/FAIL-footnote-duplicate-footnoteIDs.xml': 0.056,
    'footnotes/FAIL-footnote-invalid-element-content.xml': 0.046,
    'footnotes/FAIL-footnote-missing-footnoteID.xml': 0.055,
    'footnotes/PASS-element-ix-footnote-03.xml': 1.529,
    'footnotes/PASS-element-link-footnote-02.xml': 1.557,
    'footnotes/PASS-element-link-footnote-complete-role-defs.xml': 0.065,
    'footnotes/PASS-element-link-footnote-complete.xml': 1.571,
    'footnotes/PASS-element-link-footnote-footnoteArcrole-2.xml': 1.565,
    'footnotes/PASS-element-link-footnote-footnoteArcrole.xml': 0.063,
    'footnotes/PASS-element-link-footnote-footnoteLinkRole-2.xml': 1.602,
    'footnotes/PASS-element-link-footnote-footnoteLinkRole.xml': 0.066,
    'footnotes/PASS-element-link-footnote-footnoteRole-2.xml': 1.643,
    'footnotes/PASS-element-link-footnote-footnoteRole.xml': 0.069,
    'footnotes/PASS-element-link-footnote-nonNumeric-escaped.xml': 1.610,
    'footnotes/PASS-element-link-footnote-nonNumeric-unescaped.xml': 1.570,
    'footnotes/PASS-element-link-footnote-nothidden.xml': 1.547,
    'footnotes/PASS-element-link-footnote-resolved-uris.xml': 1.566,
    'footnotes/PASS-element-link-footnote-xhtml-content-exclude.xml': 1.598,
    'footnotes/PASS-element-link-footnote-xhtml-content.xml': 1.584,
    'footnotes/PASS-elements-footnote-and-nonNumeric-unresolvable-uris-in-exclude.xml': 1.584,
    'footnotes/PASS-footnote-any-attribute.xml': 0.065,
    'footnotes/PASS-footnote-continuation.xml': 0.064,
    'footnotes/PASS-footnote-footnoteLinkRole-multiple-output.xml': 0.129,
    'footnotes/PASS-footnote-footnoteRole-multiple-output.xml': 0.128,
    'footnotes/PASS-footnote-ix-element-content.xml': 0.064,
    'footnotes/PASS-footnote-ix-exclude-content.xml': 0.061,
    'footnotes/PASS-footnote-nested-ix-element-content.xml': 0.065,
    'footnotes/PASS-footnote-nested-xml-base-decls.xml': 0.063,
    'footnotes/PASS-footnote-on-nonFraction.xml': 0.061,
    'footnotes/PASS-footnote-order-attribute.xml': 0.062,
    'footnotes/PASS-footnote-relative-uris-object-tag.xml': 0.065,
    'footnotes/PASS-footnote-uris-with-spaces.xml': 0.062,
    'footnotes/PASS-footnote-valid-element-content.xml': 0.065,
    'footnotes/PASS-footnote-within-footnote.xml': 0.065,
    'footnotes/PASS-footnote-xml-base-xhtml-base-no-interaction.xml': 0.065,
    'footnotes/PASS-footnoteArcrole-multiple-output.xml': 0.128,
    'footnotes/PASS-footnoteRef-on-fraction.xml': 0.063,
    'footnotes/PASS-footnoteRef-on-nonNumeric.xml': 0.061,
    'footnotes/PASS-footnoteRef-on-tuple.xml': 0.061,
    'footnotes/PASS-many-to-one-footnote-complete.xml': 0.064,
    'footnotes/PASS-many-to-one-footnote-different-arcroles.xml': 0.064,
    'footnotes/PASS-many-to-one-footnotes-multiple-outputs.xml': 0.186,
    'footnotes/PASS-multiple-outputs-check-dont-have-empty-footnoteLinks.xml': 0.126,
    'footnotes/PASS-two-footnotes-multiple-output.xml': 0.128,
    'footnotes/PASS-unused-footnote.xml': 0.063,
    'format/FAIL-format-numdash-badContent.xml': 0.052,
    'format/FAIL-ix-format-undefined.xml': 3.245,
    'format/PASS-element-ix-nonFraction-ixt-num-nodecimals.xml': 1.519,
    'format/PASS-format-numdash.xml': 0.065,
    'fraction/FAIL-fraction-denominator-empty.xml': 0.052,
    'fraction/FAIL-fraction-denominator-illegal-child-node.xml': 0.053,
    'fraction/FAIL-fraction-denominator-ix-format-expanded-name-mismatch.xml': 0.054,
    'fraction/FAIL-fraction-denominator-ix-format-invalid.xml': 0.045,
    'fraction/FAIL-fraction-denominator-ix-sign-invalid.xml': 0.053,
    'fraction/FAIL-fraction-illegal-content.xml': 0.052,
    'fraction/FAIL-fraction-illegal-nesting-unitRef.xml': 0.055,
    'fraction/FAIL-fraction-illegal-nesting-xsi-nil-2.xml': 0.052,
    'fraction/FAIL-fraction-illegal-nesting-xsi-nil.xml': 0.053,
    'fraction/FAIL-fraction-illegal-nesting.xml': 0.052,
    'fraction/FAIL-fraction-ix-any-attribute.xml': 0.051,
    'fraction/FAIL-fraction-ix-contextRef-unresolvable.xml': 0.055,
    'fraction/FAIL-fraction-ix-footnoteRef-unresolvable.xml': 0.047,
    'fraction/FAIL-fraction-ix-tupleRef-attr-tuple-missing.xml': 0.053,
    'fraction/FAIL-fraction-ix-unitRef-unresolvable.xml': 0.054,
    'fraction/FAIL-fraction-missing-contextRef.xml': 0.044,
    'fraction/FAIL-fraction-missing-denominator.xml': 0.054,
    'fraction/FAIL-fraction-missing-numerator-and-denominator.xml': 0.053,
    'fraction/FAIL-fraction-missing-numerator.xml': 0.048,
    'fraction/FAIL-fraction-missing-unitRef.xml': 0.053,
    'fraction/FAIL-fraction-multiple-denominators.xml': 0.051,
    'fraction/FAIL-fraction-multiple-numerators.xml': 0.050,
    'fraction/FAIL-fraction-no-format-negative-number.xml': 0.049,
    'fraction/FAIL-fraction-numerator-denominator-non-xsi-attributes.xml': 0.055,
    'fraction/FAIL-fraction-numerator-empty.xml': 0.046,
    'fraction/FAIL-fraction-numerator-illegal-child-node.xml': 0.052,
    'fraction/FAIL-fraction-numerator-ix-format-expanded-name-mismatch.xml': 0.099,
    'fraction/FAIL-fraction-numerator-ix-format-invalid.xml': 0.048,
    'fraction/FAIL-fraction-numerator-ix-sign-invalid.xml': 0.054,
    'fraction/FAIL-fraction-rule-no-other-ixDescendants.xml': 0.045,
    'fraction/FAIL-fraction-rule-no-xbrli-attributes.xml': 0.051,
    'fraction/PASS-attribute-ix-format-denominator-01.xml': 3.623,
    'fraction/PASS-attribute-ix-format-numerator-01.xml': 3.677,
    'fraction/PASS-attribute-ix-name-fraction-01.xml': 3.701,
    'fraction/PASS-attribute-ix-scale-denominator-01.xml': 3.706,
    'fraction/PASS-attribute-ix-scale-denominator-04.xml': 3.752,
    'fraction/PASS-attribute-ix-scale-numerator-01.xml': 3.719,
    'fraction/PASS-attribute-ix-scale-numerator-04.xml': 3.696,
    'fraction/PASS-attribute-ix-sign-denominator-01.xml': 3.738,
    'fraction/PASS-attribute-ix-sign-numerator-01.xml': 3.712,
    'fraction/PASS-fraction-denominator-ix-format-expanded-name-match.xml': 0.064,
    'fraction/PASS-fraction-denominator-ix-format.xml': 0.060,
    'fraction/PASS-fraction-denominator-ix-sign-scale-valid.xml': 0.062,
    'fraction/PASS-fraction-ix-order-attr.xml': 0.065,
    'fraction/PASS-fraction-ix-target-attr.xml': 0.120,
    'fraction/PASS-fraction-ix-tupleRef-attr.xml': 0.061,
    'fraction/PASS-fraction-nesting-2.xml': 0.060,
    'fraction/PASS-fraction-nesting-3.xml': 0.062,
    'fraction/PASS-fraction-nesting-4.xml': 0.058,
    'fraction/PASS-fraction-nesting.xml': 0.062,
    'fraction/PASS-fraction-non-ix-any-attribute.xml': 0.064,
    'fraction/PASS-fraction-numerator-denominator-xsi-attributes.xml': 0.062,
    'fraction/PASS-fraction-numerator-ix-format-expanded-name-match.xml': 0.061,
    'fraction/PASS-fraction-numerator-ix-format.xml': 0.062,
    'fraction/PASS-fraction-numerator-ix-sign-valid.xml': 0.059,
    'fraction/PASS-fraction-xsi-nil.xml': 0.061,
    'fraction/PASS-ix-denominator-01.xml': 3.737,
    'fraction/PASS-ix-denominator-02.xml': 3.744,
    'fraction/PASS-ix-denominator-03.xml': 3.755,
    'fraction/PASS-ix-denominator-04.xml': 3.730,
    'fraction/PASS-ix-numerator-04.xml': 3.748,
    'fraction/PASS-simple-fraction-with-html-children.xml': 0.062,
    'fraction/PASS-simple-fraction.xml': 0.061,
    'header/FAIL-ix-header-child-of-html-header.xml': 0.054,
    'header/FAIL-misplaced-ix-element-in-context.xml': 0.054,
    'header/FAIL-missing-header.xml': 0.000,
    'header/PASS-header-content-split-over-input-docs.xml': 0.100,
    'header/PASS-header-empty.xml': 0.107,
    'header/PASS-single-ix-header-muli-input.xml': 0.099,
    'hidden/FAIL-empty-hidden.xml': 0.056,
    'hidden/FAIL-hidden-empty-tuple-content.xml': 0.046,
    'hidden/FAIL-hidden-illegal-content.xml': 0.054,
    'hidden/FAIL-hidden-incorrect-order-in-header.xml': 0.054,
    'hidden/FAIL-hidden-not-header-descendant.xml': 0.050,
    'hidden/PASS-hidden-nonFraction-content.xml': 0.063,
    'hidden/PASS-hidden-tuple-content.xml': 0.065,
    'html/FAIL-a-name-attribute.xml': 0.058,
    'html/FAIL-charset-on-meta.xml': 0.054,
    'html/FAIL-empty-class-attribute.xml': 0.057,
    'ids/FAIL-id-triplication.xml': 0.047,
    'ids/FAIL-non-unique-id-context.xml': 0.095,
    'ids/FAIL-non-unique-id-footnote.xml': 0.053,
    'ids/FAIL-non-unique-id-fraction.xml': 0.089,
    'ids/FAIL-non-unique-id-nonFraction.xml': 0.094,
    'ids/FAIL-non-unique-id-nonNumeric.xml': 0.091,
    'ids/FAIL-non-unique-id-references.xml': 0.054,
    'ids/FAIL-non-unique-id-tuple.xml': 0.096,
    'ids/FAIL-non-unique-id-unit.xml': 0.088,
    'multiIO/FAIL-multi-input-duplicate-context-ids.xml': 0.092,
    'multiIO/FAIL-multi-input-duplicate-unit-ids.xml': 0.093,
    'multiIO/FAIL-two-inputs-each-with-error.xml': 0.091,
    'multiIO/FAIL-two-nonIXBRL-inputs.xml': 0.001,
    'multiIO/PASS-double-input-single-output.xml': 0.108,
    'multiIO/PASS-ix-references-06.xml': 3.289,
    'multiIO/PASS-ix-references-07.xml': 3.291,
    'multiIO/PASS-multiple-input-multiple-output.xml': 0.211,
    'multiIO/PASS-single-input-double-output.xml': 0.126,
    'multiIO/PASS-single-input.xml': 0.060,
    'nonFraction/FAIL-nonFraction-IXBRLelement-content.xml': 0.057,
    'nonFraction/FAIL-nonFraction-any-ix-attribute.xml': 0.056,
    'nonFraction/FAIL-nonFraction-decimals-and-precision-attrs.xml': 0.140,
    'nonFraction/FAIL-nonFraction-double-nesting.xml': 0.057,
    'nonFraction/FAIL-nonFraction-empty-content.xml': 0.047,
    'nonFraction/FAIL-nonFraction-empty-without-xsi-nil.xml': 0.056,
    'nonFraction/FAIL-nonFraction-invalid-sign-attr.xml': 0.052,
    'nonFraction/FAIL-nonFraction-ix-format-expanded-name-mismatch.xml': 0.056,
    'nonFraction/FAIL-nonFraction-ix-format-invalid-minus-sign.xml': 0.050,
    'nonFraction/FAIL-nonFraction-ix-format-invalid.xml': 0.055,
    'nonFraction/FAIL-nonFraction-missing-context-attr.xml': 0.054,
    'nonFraction/FAIL-nonFraction-missing-name-attr.xml': 0.055,
    'nonFraction/FAIL-nonFraction-missing-unit-attr.xml': 0.048,
    'nonFraction/FAIL-nonFraction-mixed-nesting-2.xml': 0.055,
    'nonFraction/FAIL-nonFraction-mixed-nesting-3.xml': 0.051,
    'nonFraction/FAIL-nonFraction-mixed-nesting.xml': 0.056,
    'nonFraction/FAIL-nonFraction-neither-decimals-nor-precision-attrs.xml': 0.055,
    'nonFraction/FAIL-nonFraction-nesting-format-mismatch-2.xml': 0.053,
    'nonFraction/FAIL-nonFraction-nesting-format-mismatch.xml': 0.053,
    'nonFraction/FAIL-nonFraction-nesting-scale-mismatch-2.xml': 0.054,
    'nonFraction/FAIL-nonFraction-nesting-scale-mismatch.xml': 0.054,
    'nonFraction/FAIL-nonFraction-nesting-unitRef-mismatch.xml': 0.055,
    'nonFraction/FAIL-nonFraction-nesting-xsi-nil.xml': 0.057,
    'nonFraction/FAIL-nonFraction-nil-attr-false.xml': 0.054,
    'nonFraction/FAIL-nonFraction-nil-decimal-conflict.xml': 0.057,
    'nonFraction/FAIL-nonFraction-no-format-negative-number.xml': 0.070,
    'nonFraction/FAIL-nonFraction-unresolvable-ix-tupleRef-attr.xml': 0.047,
    'nonFraction/FAIL-nonfraction-rule-no-xbrli-attributes.xml': 0.053,
    'nonFraction/FAIL-unresolvable-contextRef.xml': 0.056,
    'nonFraction/FAIL-unresolvable-unitRef.xml': 0.056,
    'nonFraction/PASS-attribute-ix-format-nonFraction-01.xml': 3.632,
    'nonFraction/PASS-attribute-ix-name-nonFraction-01.xml': 1.717,
    'nonFraction/PASS-attribute-ix-scale-nonFraction-01.xml': 3.763,
    'nonFraction/PASS-attribute-ix-scale-nonFraction-04.xml': 3.736,
    'nonFraction/PASS-attribute-ix-sign-nonFraction-01.xml': 3.749,
    'nonFraction/PASS-element-ix-nonFraction-complete.xml': 1.576,
    'nonFraction/PASS-element-ix-nonFraction-ixt-numcomma.xml': 1.586,
    'nonFraction/PASS-element-ix-nonFraction-ixt-numcommadot.xml': 1.548,
    'nonFraction/PASS-element-ix-nonFraction-ixt-numdash.xml': 1.541,
    'nonFraction/PASS-element-ix-nonFraction-ixt-numdotcomma.xml': 1.543,
    'nonFraction/PASS-element-ix-nonFraction-ixt-numspacecomma.xml': 1.559,
    'nonFraction/PASS-element-ix-nonFraction-ixt-numspacedot.xml': 1.557,
    'nonFraction/PASS-nonFraction-any-attribute.xml': 0.065,
    'nonFraction/PASS-nonFraction-comments.xml': 0.065,
    'nonFraction/PASS-nonFraction-decimals-attr.xml': 0.063,
    'nonFraction/PASS-nonFraction-ix-format-expanded-name-match.xml': 0.064,
    'nonFraction/PASS-nonFraction-ix-format-valid.xml': 0.065,
    'nonFraction/PASS-nonFraction-ix-order-attr.xml': 0.061,
    'nonFraction/PASS-nonFraction-ix-target-attr.xml': 0.116,
    'nonFraction/PASS-nonFraction-ix-tupleRef-attr.xml': 0.061,
    'nonFraction/PASS-nonFraction-nesting-2.xml': 0.059,
    'nonFraction/PASS-nonFraction-nesting-formats-2.xml': 0.063,
    'nonFraction/PASS-nonFraction-nesting-formats.xml': 0.115,
    'nonFraction/PASS-nonFraction-nesting-scale.xml': 0.061,
    'nonFraction/PASS-nonFraction-nesting.xml': 0.060,
    'nonFraction/PASS-nonFraction-precision-attr.xml': 0.056,
    'nonFraction/PASS-nonFraction-processing-instructions.xml': 0.062,
    'nonFraction/PASS-nonFraction-valid-scale-attr.xml': 0.059,
    'nonFraction/PASS-nonFraction-valid-sign-attr.xml': 0.062,
    'nonFraction/PASS-nonFraction-valid-sign-format-attr.xml': 0.060,
    'nonFraction/PASS-nonFraction-xsi-nil-attr.xml': 0.061,
    'nonFraction/PASS-simple-nonFraction.xml': 0.061,
    'nonNumeric/FAIL-element-ix-nonNumeric-escape-01.xml': 1.300,
    'nonNumeric/FAIL-nonNumeric-any-ix-attribute.xml': 0.054,
    'nonNumeric/FAIL-nonNumeric-empty-with-format.xml': 0.052,
    'nonNumeric/FAIL-nonNumeric-illegal-null-namespace-attr.xml': 0.050,
    'nonNumeric/FAIL-nonNumeric-invalid-ix-format-attr.xml': 0.047,
    'nonNumeric/FAIL-nonNumeric-ix-format-attr-wrong-namespace-binding.xml': 0.051,
    'nonNumeric/FAIL-nonNumeric-missing-contextRef-attr.xml': 0.052,
    'nonNumeric/FAIL-nonNumeric-missing-name-attr.xml': 0.051,
    'nonNumeric/FAIL-nonNumeric-no-xbrli-attributes.xml': 0.044,
    'nonNumeric/FAIL-nonNumeric-unresolvable-ix-contextRef.xml': 0.050,
    'nonNumeric/FAIL-nonNumeric-unresolvable-ix-tupleRef-attr.xml': 0.052,
    'nonNumeric/PASS-attribute-ix-extension-illegalPlacement-01.xml': 3.557,
    'nonNumeric/PASS-attribute-ix-name-nonNumeric-01.xml': 1.666,
    'nonNumeric/PASS-element-ix-nonNumeric-complete.xml': 1.507,
    'nonNumeric/PASS-element-ix-nonNumeric-escape-02.xml': 1.506,
    'nonNumeric/PASS-element-ix-nonNumeric-escape-03.xml': 1.507,
    'nonNumeric/PASS-element-ix-nonNumeric-escape-04.xml': 1.518,
    'nonNumeric/PASS-element-ix-nonNumeric-escape-05.xml': 1.528,
    'nonNumeric/PASS-element-ix-nonNumeric-escape-06.xml': 1.487,
    'nonNumeric/PASS-element-ix-nonNumeric-escape-07.xml': 1.490,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-datedoteu-01.xml': 1.493,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-datedotus-01.xml': 1.498,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-datedotus-02.xml': 1.502,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-datedotus-03.xml': 1.513,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-datelongdaymonthuk-01.xml': 1.516,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-datelongmonthdayus-01.xml': 1.526,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-datelongmonthyear-01.xml': 1.492,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-datelonguk-01.xml': 1.510,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-datelonguk-02.xml': 1.505,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-datelongus-01.xml': 1.530,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-datelongus-02.xml': 1.516,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-datelongyearmonth-01.xml': 1.531,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-dateshortdaymonthuk-01.xml': 1.506,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-dateshortmonthdayus-01.xml': 1.537,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-dateshortmonthyear-01.xml': 1.489,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-dateshortuk-01.xml': 1.496,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-dateshortuk-02.xml': 1.504,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-dateshortus-01.xml': 1.507,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-dateshortus-02.xml': 1.510,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-dateshortyearmonth-01.xml': 1.483,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-dateslashdaymontheu-01.xml': 1.487,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-dateslasheu-01.xml': 1.490,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-dateslasheu-02.xml': 1.495,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-dateslasheu-03.xml': 1.520,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-dateslashmonthdayus-01.xml': 1.525,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-dateslashus-01.xml': 1.519,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-dateslashus-02.xml': 1.525,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-dateslashus-03.xml': 1.493,
    'nonNumeric/PASS-element-ix-nonNumeric-ixt-dateslashus-04.xml': 1.499,
    'nonNumeric/PASS-element-ordering.xml': 1.496,
    'nonNumeric/PASS-nonNumeric-any-attribute.xml': 0.061,
    'nonNumeric/PASS-nonNumeric-empty-not-xsi-nil.xml': 0.058,
    'nonNumeric/PASS-nonNumeric-escape-with-html-base.xml': 1.502,
    'nonNumeric/PASS-nonNumeric-ix-format-attr-expanded-name-match.xml': 0.060,
    'nonNumeric/PASS-nonNumeric-ix-format-attr.xml': 0.058,
    'nonNumeric/PASS-nonNumeric-ix-order-attr.xml': 0.068,
    'nonNumeric/PASS-nonNumeric-ix-target-attr.xml': 0.119,
    'nonNumeric/PASS-nonNumeric-ix-tupleRef-attr.xml': 0.057,
    'nonNumeric/PASS-nonNumeric-nesting-in-exclude.xml': 0.058,
    'nonNumeric/PASS-nonNumeric-nesting-numerator.xml': 0.058,
    'nonNumeric/PASS-nonNumeric-nesting-text-in-exclude.xml': 0.059,
    'nonNumeric/PASS-nonNumeric-nesting.xml': 0.107,
    'nonNumeric/PASS-nonNumeric-xsi-nil.xml': 0.054,
    'nonNumeric/PASS-nonNumeric.xml': 0.059,
    'references/FAIL-empty-references.xml': 0.039,
    'references/FAIL-ix-references-03.xml': 1.277,
    'references/FAIL-ix-references-08.xml': 1.388,
    'references/FAIL-ix-references-09.xml': 1.405,
    'references/FAIL-ix-references-namespace-bindings-01.xml': 1.384,
    'references/FAIL-ix-references-namespace-bindings-02.xml': 1.404,
    'references/FAIL-ix-references-namespace-bindings-03.xml': 1.394,
    'references/FAIL-ix-references-namespace-bindings-04.xml': 1.418,
    'references/FAIL-ix-references-rule-multiple-attributes-sameValue.xml': 1.317,
    'references/FAIL-ix-references-rule-multiple-id.xml': 1.312,
    'references/FAIL-missing-references-for-all-target-documents.xml': 0.044,
    'references/FAIL-missing-references.xml': 0.035,
    'references/FAIL-references-illegal-content.xml': 0.050,
    'references/FAIL-references-illegal-location.xml': 0.043,
    'references/FAIL-references-illegal-order-in-header.xml': 0.051,
    'references/PASS-element-ix-references-01.xml': 1.489,
    'references/PASS-ix-references-01.xml': 1.500,
    'references/PASS-ix-references-02.xml': 1.508,
    'references/PASS-ix-references-04.xml': 1.623,
    'references/PASS-ix-references-05.xml': 1.617,
    'references/PASS-ix-references-rule-multiple-matched-target.xml': 1.651,
    'references/PASS-ix-references-rule-multiple-xmlBase.xml': 0.060,
    'references/PASS-references-copy-non-ix-attrs.xml': 0.059,
    'references/PASS-references-ix-target-attr.xml': 0.113,
    'references/PASS-simple-linkbaseRef.xml': 0.054,
    'references/PASS-simple-schemaRef.xml': 0.059,
    'references/PASS-single-references-multi-input.xml': 0.096,
    'relationships/FAIL-relationship-cross-duplication.xml': 0.047,
    'relationships/FAIL-relationship-mixes-footnote-with-explanatory-fact.xml': 0.053,
    'relationships/FAIL-relationship-with-no-namespace-attribute.xml': 0.048,
    'relationships/FAIL-relationship-with-xbrli-attribute.xml': 0.050,
    'relationships/FAIL-relationship-with-xlink-attribute.xml': 0.043,
    'relationships/PASS-explanatory-fact-copy-to-owner-target.xml': 0.110,
    'relationships/PASS-explanatory-fact-cycle.xml': 0.062,
    'relationships/PASS-explanatory-fact-not-hidden.xml': 0.057,
    'relationships/PASS-explanatory-fact.xml': 0.056,
    'relationships/PASS-relationship-to-multiple-explanatory-facts-multiple-outputs.xml': 0.117,
    'relationships/PASS-relationship-to-multiple-explanatory-facts.xml': 0.055,
    'relationships/PASS-relationship-with-xml-base.xml': 0.053,
    'relationships/PASS-tuple-footnotes.xml': 0.057,
    'resources/FAIL-context-without-id.xml': 0.050,
    'resources/FAIL-missing-resources.xml': 0.048,
    'resources/FAIL-unit-without-id.xml': 0.053,
    'resources/PASS-empty-resources.xml': 0.056,
    'resources/PASS-simple-arcroleRef.xml': 0.057,
    'resources/PASS-simple-roleRef.xml': 0.062,
    'specificationExamples/PASS-section-10.3-example-1.xml': 1.510,
    'specificationExamples/PASS-section-11.3-example-2.xml': 1.504,
    'specificationExamples/PASS-section-15.1-example-3.xml': 1.629,
    'specificationExamples/PASS-section-15.1-example-4.xml': 1.625,
    'transformations/FAIL-invalid-long-month.xml': 0.053,
    'transformations/FAIL-invalid-short-month.xml': 0.043,
    'transformations/FAIL-unrecognised-schema-type.xml': 0.098,
    'transformations/PASS-sign-attribute-on-nonFraction-positive-input.xml': 0.061,
    'tuple/FAIL-badly-formatted-order-attr.xml': 0.052,
    'tuple/FAIL-badly-nested-tuples.xml': 0.051,
    'tuple/FAIL-duplicate-order-and-value-but-not-attributes.xml': 0.051,
    'tuple/FAIL-duplicate-tuple-id-different-input-docs.xml': 0.088,
    'tuple/FAIL-duplicate-tuple-id.xml': 0.056,
    'tuple/FAIL-duplicate-tuple-order-different-values.xml': 0.051,
    'tuple/FAIL-illegal-element-nested.xml': 0.051,
    'tuple/FAIL-illegal-element.xml': 0.042,
    'tuple/FAIL-missing-descendants.xml': 0.049,
    'tuple/FAIL-nested-tuple-empty.xml': 0.051,
    'tuple/FAIL-order-attr-denominator.xml': 0.049,
    'tuple/FAIL-order-attr-inNonTuple.xml': 0.045,
    'tuple/FAIL-order-attr-numerator.xml': 0.048,
    'tuple/FAIL-ordering-order-duplicate-stringUnequal.xml': 0.052,
    'tuple/FAIL-ordering-order-duplicate.xml': 0.047,
    'tuple/FAIL-ordering-partially-missing.xml': 0.053,
    'tuple/FAIL-orphaned-tuple-content.xml': 0.052,
    'tuple/FAIL-tuple-any-ix-attribute.xml': 0.053,
    'tuple/FAIL-tuple-content-in-different-targets-tuple-not-in-default.xml': 0.049,
    'tuple/FAIL-tuple-content-in-different-targets.xml': 0.047,
    'tuple/FAIL-tuple-cycle-by-tupleRef.xml': 0.050,
    'tuple/FAIL-tuple-cycle-child.xml': 0.048,
    'tuple/FAIL-tuple-cycle-grandchildren.xml': 0.064,
    'tuple/FAIL-tuple-empty-no-ix-tupleID.xml': 0.048,
    'tuple/FAIL-tuple-empty.xml': 0.052,
    'tuple/FAIL-tuple-missing-name-attr.xml': 0.044,
    'tuple/FAIL-tuple-no-xbrli-attributes.xml': 0.052,
    'tuple/FAIL-tuple-unresolvable-footnoteRef-attr.xml': 0.051,
    'tuple/FAIL-tuple-xsi-nil-with-tuple-ref.xml': 0.048,
    'tuple/PASS-attribute-ix-name-tuple-01.xml': 1.585,
    'tuple/PASS-duplicate-order-same-ws-normalized-value-with-html.xml': 0.058,
    'tuple/PASS-duplicate-order-same-ws-normalized-value.xml': 0.060,
    'tuple/PASS-element-ix-tuple-complete.xml': 1.512,
    'tuple/PASS-element-tuple-reference-multiInput.xml': 0.099,
    'tuple/PASS-element-tuple-reference.xml': 0.061,
    'tuple/PASS-exotic-tuple-order.xml': 0.056,
    'tuple/PASS-nested-tuple-ix-order-no-tupleRef.xml': 0.060,
    'tuple/PASS-nested-tuple-nonEmpty.xml': 0.061,
    'tuple/PASS-nested-tuple.xml': 0.059,
    'tuple/PASS-nonFraction-nesting-reference-conflict.xml': 0.062,
    'tuple/PASS-ordering-references-nesting-order.xml': 0.057,
    'tuple/PASS-singleton-tuple.xml': 0.059,
    'tuple/PASS-tuple-all-content-nested-noTupleID.xml': 0.063,
    'tuple/PASS-tuple-any-attribute.xml': 0.107,
    'tuple/PASS-tuple-ix-target-attr.xml': 0.126,
    'tuple/PASS-tuple-nested-nonNumeric.xml': 0.057,
    'tuple/PASS-tuple-nesting-reference-conflict.xml': 0.058,
    'tuple/PASS-tuple-nonInteger-ordering-nested.xml': 0.062,
    'tuple/PASS-tuple-ordering-nested.xml': 0.057,
    'tuple/PASS-tuple-scope-inverted-siblings.xml': 0.060,
    'tuple/PASS-tuple-scope-inverted.xml': 0.058,
    'tuple/PASS-tuple-scope-nested-nonNumeric.xml': 0.060,
    'tuple/PASS-tuple-scope-nonNumeric.xml': 0.060,
    'tuple/PASS-tuple-xsi-nil.xml': 0.059,
    'xmllang/FAIL-xml-lang-not-in-scope-for-footnote.xml': 0.051,
    'xmllang/FAIL-xml-lang-on-ix-hidden-and-on-footnote.xml': 0.050,
    'xmllang/FAIL-xml-lang-on-ix-hidden.xml': 0.059,
    'xmllang/PASS-direct-xml-lang-not-overidden.xml': 0.058,
    'xmllang/PASS-xml-lang-on-xhtml.xml': 0.059,
}.items()}

ZIP_PATH = 'inlineXBRL-1.1-conformanceSuite-2020-04-08.zip'
# needs to be extracted because arelle can't load a taxonomy package ZIP from within a ZIP
EXTRACTED_PATH = ZIP_PATH.replace('.zip', '')
config = ConformanceSuiteConfig(
    approximate_relative_timing=TIMING,
    args=[
        '--packages', os.path.join(CONFORMANCE_SUITE_PATH_PREFIX, EXTRACTED_PATH, 'inlineXBRL-1.1-conformanceSuite-2020-04-08/schemas/www.example.com.zip'),
    ],
    capture_warnings=False,
    extract_path=EXTRACTED_PATH,
    file='inlineXBRL-1.1-conformanceSuite-2020-04-08/index.xml',
    info_url='https://specifications.xbrl.org/work-product-index-inline-xbrl-inline-xbrl-1.1.html',
    local_filepath=ZIP_PATH,
    name=PurePath(__file__).stem,
    network_or_cache_required=False,
    plugins=frozenset({'inlineXbrlDocumentSet', '../../tests/plugin/testcaseIxExpectedHtmlFixup.py'}),
    public_download_url='https://www.xbrl.org/2020/inlineXBRL-1.1-conformanceSuite-2020-04-08.zip',
    shards=4,
)



from pathlib import PurePath
from tests.integration_tests.validation.conformance_suite_config import ConformanceSuiteConfig

TIMING = {
    'br_kvk/2-04-testcase.xml': 6.364,
    'br_kvk/3-01-testcase.xml': 13.398,
    'br_kvk/4-07-testcase.xml': 6.878,
    'br_kvk/4-10-testcase.xml': 6.826,
    'br_kvk/4-12-testcase.xml': 7.250,
    'br_kvk/4-16-testcase.xml': 7.260,
    'br_kvk/4-17-testcase.xml': 1.161,
    'br_kvk/4-20-testcase.xml': 6.778,
    'fg_nl/04-testcase.xml': 0.011,
    'fg_nl/05-testcase.xml': 0.038,
    'fg_nl/09-testcase.xml': 0.343,
    'fg_nl/11-testcase.xml': 0.011,
    'fr_kvk/1-01-testcase.xml': 6.457,
    'fr_kvk/2-01-testcase.xml': 14.355,
    'fr_kvk/2-03-testcase.xml': 0.003,
    'fr_kvk/5-01-testcase.xml': 7.117,
    'fr_kvk/5-02-testcase.xml': 7.063,
    'fr_nl/1-01-testcase.xml': 0.007,
    'fr_nl/1-02-testcase.xml': 0.060,
    'fr_nl/1-03-testcase.xml': 0.011,
    'fr_nl/1-04-testcase.xml': 0.004,
    'fr_nl/1-05-testcase.xml': 0.080,
    'fr_nl/1-06-testcase.xml': 0.003,
    'fr_nl/2-03-testcase.xml': 0.296,
    'fr_nl/2-04-testcase.xml': 0.012,
    'fr_nl/2-05-testcase.xml': 0.011,
    'fr_nl/2-06-testcase.xml': 0.057,
    'fr_nl/2-07-testcase.xml': 1.979,
    'fr_nl/3-01-testcase.xml': 0.841,
    'fr_nl/3-02-testcase.xml': 0.799,
    'fr_nl/3-03-testcase.xml': 0.801,
    'fr_nl/3-04-testcase.xml': 2.512,
    'fr_nl/4-01-testcase.xml': 1.697,
    'fr_nl/4-02-testcase.xml': 0.873,
    'fr_nl/5-01-testcase.xml': 1.719,
    'fr_nl/5-03-testcase.xml': 0.873,
    'fr_nl/5-06-testcase.xml': 0.843,
    'fr_nl/5-11-testcase.xml': 0.057,
    'fr_nl/6-01-testcase.xml': 0.854,
}

config = ConformanceSuiteConfig(
    approximate_relative_timing=TIMING,
    args=[
        '--disclosureSystem', 'NT17-preview',
    ],
    file='index.xml',
    info_url='https://sbr-nl.nl/sites/default/files/bestanden/taxonomie/SBR%20Filing%20Rules%20NT17%20-%2020220301__.pdf',
    local_filepath='nl_nt17',
    name=PurePath(__file__).stem,
    plugins=frozenset({'validate/NL'}),
    shards=4,
)



"""
See COPYRIGHT.md for copyright information.
"""
from __future__ import annotations

from dataclasses import dataclass
from dataclasses import field
from enum import Enum
from functools import cached_property
from regex.regex import Pattern
from typing import Any, Dict, Tuple, cast

import regex as re

from arelle.ModelInstanceObject import ModelFact
from arelle.ModelXbrl import ModelXbrl

# Error codes
CO_ABRID = 'Co.Abrid'
CO_AUDIT = 'Co.Audit'
CO_AUDIT_NR = 'Co.AuditNR'
CO_DIR_REP = 'Co.DirReport'
CO_DIR_RESP = 'Co.DirResp'
CO_MED_CO = 'Co.MedCo'
CO_MICRO = 'Co.Micro'
CO_MISSING_ELEMENT = 'Co.MissingElement'
CO_PROF_LOSS = 'Co.ProfLoss'
CO_QUAL_AUDIT = 'Co.QualAudit'
CO_SM_CO = 'Co.SmCo'
CO_SEC_477 = 'Co.Sec477'
CO_SEC_480 = 'Co.Sec480'
LP_ABRID = 'Lp.Abrid'
LP_AUDIT = 'Lp.Audit'
LP_MED_LP = 'Lp.MedLp'
LP_MEM_RESP = 'Lp.MemResp'
LP_MICRO = 'Lp.Micro'
LP_PROF_LOSS = 'Lp.ProfLoss'
LP_QUAL_AUDIT = 'Lp.QualAudit'
LP_SEC_477 = 'Lp.Sec477'
LP_SEC_480 = 'Lp.Sec480'
LP_SM_LP = 'Lp.SmLp'

# Concept local names
CONCEPT_ABRIDGED_ACCOUNTS = 'AbridgedAccounts'
CONCEPT_ABBREVIATED_ACCOUNTS = 'AbbreviatedAccounts'
CONCEPT_AUDITED = 'Audited'
CONCEPT_APPLICABLE_LEGISLATION = 'ApplicableLegislation'
CONCEPT_APPLICABLE_LEGISLATION_DIMENSION = 'ApplicableLegislationDimension'
CONCEPT_ACCOUNTING_STANDARDS_APPLIED = 'AccountingStandardsApplied'
CONCEPT_ACCOUNTING_STANDARDS_DIMENSION = 'AccountingStandardsDimension'
CONCEPT_ACCOUNTS_STATUS = 'AccountsStatusAuditedOrUnaudited'
CONCEPT_ACCOUNTS_STATUS_DIMENSION = 'AccountsStatusDimension'
CONCEPT_ACCOUNTS_TYPE_FULL_OR_ABBREVIATED = 'AccountsTypeFullOrAbbreviated'  # DEPRECATED IN 2022+ taxonomies.  No replacement yet.
CONCEPT_ACCOUNTS_TYPE_DIMENSION = 'AccountsTypeDimension'
CONCEPT_DATE_AUDITOR_REPORT = 'DateAuditorsReport'
CONCEPT_DATE_SIGNING_DIRECTOR_REPORT = 'DateSigningDirectorsReport'
CONCEPT_DIRECTOR_SIGNING_DIRECTORS_REPORT = 'DirectorSigningDirectorsReport'
CONCEPT_ENTITY_DORMANT = 'EntityDormantTruefalse'
CONCEPT_ENTITY_TRADING_STATUS = 'EntityTradingStatus'
CONCEPT_ENTITY_TRADING_STATUS_DIMENSION = 'EntityTradingStatusDimension'
CONCEPT_LANGUAGES_DIMENSION = 'LanguagesDimension'
CONCEPT_MEDIUM_COMPANY = 'StatementThatCompanyHasPreparedAccountsUnderProvisionsRelatingToMedium-sizedCompanies'
CONCEPT_MEDIUM_COMPANIES_REGIME_FOR_ACCOUNTS = 'Medium-sizedCompaniesRegimeForAccounts'
CONCEPT_MICRO_ENTITIES = 'Micro-entities'
CONCEPT_NAME_INDIVIDUAL_AUDITOR = 'NameIndividualAuditor'
CONCEPT_NAME_ENTITY_AUDITORS = 'NameEntityAuditors'
CONCEPT_NAME_SENIOR_STATUTORY_AUDITOR = 'NameSeniorStatutoryAuditor'
CONCEPT_OPINION_AUDITORS_ON_ENTITY = 'OpinionAuditorsOnEntity'
CONCEPT_LEGAL_FORM_ENTIY = 'LegalFormEntity'
CONCEPT_LEGAL_FORM_ENTIY_DIMENSION = 'LegalFormEntityDimension'
CONCEPT_LLP = 'LimitedLiabilityPartnershipLLP'
CONCEPT_PROFIT_LOSS = 'ProfitLoss'
CONCEPT_REPORT_PRINCIPAL_LANGUAGE = 'ReportPrincipalLanguage'
CONCEPT_SCOPE_ACCOUNTS = 'ScopeAccounts'
CONCEPT_SCOPE_ACCOUNTS_DIMENSION = 'ScopeAccountsDimension'
CONCEPT_STATEMENT_PROVIDED = 'StatementOnQualityCompletenessInformationProvidedToAuditors'
CONCEPT_SMALL_COMPANY_REGIME_FOR_ACCOUNTS = 'SmallCompaniesRegimeForAccounts'
CONCEPT_GROUP_ACCOUNTS_ONLY = 'GroupAccountsOnly'
CONCEPT_CONSOLIDATED_GROUP_COMPANY_ACCOUNTS = 'ConsolidatedGroupCompanyAccounts'
CONCEPT_WELSH = 'Welsh'

# Map of error code > concept local name > tuple of pairings of descriptions and regex patterns
TEXT_VALIDATION_PATTERNS: dict[str, dict[str, tuple[tuple[str, re.regex.Pattern[str]], ...]]] = {
    CO_ABRID: {
        'StatementThatMembersHaveAgreedToPreparationAbridgedAccountsUnderSection444CompaniesAct2006': (
            (
                'Members" or "Member", then "agreed" or "consented", then "preparation", then "abridged")',
                re.compile(r".*(Members|Member).*(agreed|consented).*preparation.*abridged.*"),
            ),
            (
                '"Aelodau" or "Aelod", then "wedi cytuno" or "cydsynio", then "paratoi", then "talfyredig")',
                re.compile(r".*(Aelodau|Aelod).*(wedi cytuno|cydsynio).*paratoi.*talfyredig.*"),
            ),
        ),
    },
    CO_AUDIT_NR: {
        'StatementThatMembersHaveNotRequiredCompanyToObtainAnAudit': (
            (
                '"member has" or "members have" then "not required the company to obtain an audit"',
                re.compile(r".*([Mm]ember has|[Mm]embers have) not required the company to obtain an audit.*"),
            ),
            (
                '"aelodau heb ei gwneud yn ofynnol i\'r cwmni gael archwiliad" OR "aelod heb ei gwneud yn ofynnol i\'r cwmni gael archwiliad"',
                re.compile(r".*(aelodau heb ei gwneud yn ofynnol i\'r cwmni gael archwiliad|aelod heb ei gwneud yn ofynnol i\'r cwmni gael archwiliad).*"),
            ),
        ),
    },
    CO_DIR_RESP: {
        'StatementThatDirectorsAcknowledgeTheirResponsibilitiesUnderCompaniesAct': (
            (
                '"Directors acknowledge" or "Director acknowledges", then "responsibilities", then "Companies Act 2006" OR "the Act"',
                re.compile(r".*(Director acknowledges|Directors acknowledge).*responsibilities.*(Companies Act 2006|the Act).*"),
            ),
            (
                '"Cyfarwyddwyr yn cydnabod" or "Cyfarwyddwr yn cydnabod", then "cyfrifoldebau", then "Ddeddf Cwmnïau 2006" or "y ddeddf"',
                re.compile(r".*(Cyfarwyddwyr yn cydnabod|Cyfarwyddwr yn cydnabod).*cyfrifoldebau.*(Ddeddf Cwmnïau 2006|y ddeddf).*"),
            ),
        ),
    },
    CO_MICRO: {
        'StatementThatAccountsHaveBeenPreparedInAccordanceWithProvisionsSmallCompaniesRegime': (
            (
                '"Prepared", then "in accordance with", then "provisions", then "micro"',
                re.compile(r".*Prepared.*in accordance with.*provisions.*micro.*"),
            ),
            (
                '"wedi eu paratoi", then "yn unol â", then "darpariaethau", then "micro"',
                re.compile(r".*wedi eu paratoi.*yn unol â.*darpariaethau.*micro.*"),
            ),
        ),
    },
    CO_SEC_477: {
        'StatementThatCompanyEntitledToExemptionFromAuditUnderSection477CompaniesAct2006RelatingToSmallCompanies': (
            (
                '"Exempt" or "Exemption", then later "section 477 of the Companies Act 2006"',
                re.compile(r".*(Exempt|Exemption).*section 477 of the Companies Act 2006.*"),
            ),
            (
                '"Wedi\'i eithrio" or "Eithriad", then "adran 477 o Ddeddf Cwmnïau 2006"',
                re.compile(r".*(Wedi'i eithrio|Eithriad).*adran 477 o Ddeddf Cwmnïau 2006.*"),
            ),
        ),
    },
    CO_SEC_480: {
        'StatementThatCompanyEntitledToExemptionFromAuditUnderSection480CompaniesAct2006RelatingToDormantCompanies': (
            (
                '"Exempt" or "Exemption", then later "section 480 of the Companies Act 2006"',
                re.compile(r".*(Exempt|Exemption).*section 480 of the Companies Act 2006.*"),
            ),
            (
                '"Wedi\'i eithrio" or "Eithriad", then "adran 480 o Ddeddf Cwmnïau 2006"',
                re.compile(r".*(Wedi'i eithrio|Eithriad).*adran 480 o Ddeddf Cwmnïau 2006.*"),
            ),
        ),
    },
    CO_SM_CO: {
        'StatementThatAccountsHaveBeenPreparedInAccordanceWithProvisionsSmallCompaniesRegime': (
            (
                '"Prepared in accordance with", then "provisions", then "small companies"',
                re.compile(r".*Prepared in accordance with.*provisions.*small companies.*"),
            ),
            (
                '"Paratowyd yn unol â", then "darpariaethau", then "cwmnïau bach"',
                re.compile(r".*Paratowyd yn unol â.*darpariaethau.*cwmnïau bach.*"),
            ),
        ),
    },
    LP_ABRID: {
        'StatementThatMembersHaveAgreedToPreparationAbridgedAccountsUnderSection444CompaniesAct2006': (
            (
                'Members" or "Member", then "agreed" or "consented", then "preparation", then "abridged")',
                re.compile(r".*(Members|Member).*(agreed|consented).*preparation.*abridged.*"),
            ),
            (
                '"Aelodau" or "Aelod", then "wedi cytuno" or "cydsynio", then "paratoi", then "talfyredig")',
                re.compile(r".*(Aelodau|Aelod).*(wedi cytuno|cydsynio).*paratoi.*talfyredig.*"),
            ),
        ),
    },
    LP_MEM_RESP: {
            'StatementThatDirectorsAcknowledgeTheirResponsibilitiesUnderCompaniesAct': (
                (
                    '"Members acknowledge" or "Member acknowledges", then "responsibilities", then  "Companies Act 2006" or "the Act", then "Limited Liability Partnership" or "LLP"',
                    re.compile(r".*(Members acknowledge|Member acknowledges).*responsibilities.*(Companies Act 2006|the Act).*(Limited Liability Partnership|LLP).*"),
                ),
                (
                    '"Aelodau\'n cydnabod" or "Aelod yn cydnabod", then "cyfrifoldebau", then "Ddeddf Cwmnïau 2006" or "y Ddeddf", then "Partneriaeth Atebolrwydd Cyfyngedig" or "PAC"',
                    re.compile(r".*(Aelodau'n cydnabod|Aelod yn cydnabod).*cyfrifoldebau.*(Ddeddf Cwmnïau 2006|y Ddeddf).*(Partneriaeth Atebolrwydd Cyfyngedig|PAC).*"),
                ),
            ),
    },
    LP_MICRO: {
        'StatementThatAccountsHaveBeenPreparedInAccordanceWithProvisionsSmallCompaniesRegime': (
            (
                '"Prepared", then "in accordance with", then "provisions", then "micro"',
                re.compile(r".*Prepared.*in accordance with.*provisions.*micro.*"),
            ),
            (
                '"wedi eu paratoi", then "yn unol â", then "darpariaethau", then "micro"',
                re.compile(r".*wedi eu paratoi.*yn unol â.*darpariaethau.*micro.*"),
            ),
        ),
    },
    LP_SEC_477: {
        'StatementThatCompanyEntitledToExemptionFromAuditUnderSection477CompaniesAct2006RelatingToSmallCompanies': (
            (
                '"Exempt" OR "Exemption", then "section 477 of the Companies Act 2006" then, "Limited Liability Partnership" OR "LLP")',
                re.compile(r".*(Exempt|Exemption).*section 477 of the Companies Act 2006.*(Limited Liability Partnership|LLP).*"),
            ),
            (
                '"Wedi\'i eithrio" or "Eithriad", then "adran 477 o Ddeddf Cwmnïau 2006", then "Partneriaeth Atebolrwydd Cyfyngedig" OR "PAC"',
                re.compile(r".*(Wedi'i eithrio|Eithriad).*adran 477 o Ddeddf Cwmnïau 2006.*(Partneriaeth Atebolrwydd Cyfyngedig|PAC).*"),
            ),
        ),
    },
    LP_SEC_480: {
            'StatementThatCompanyEntitledToExemptionFromAuditUnderSection480CompaniesAct2006RelatingToDormantCompanies': (
                (
                    '"Exempt" or "Exemption", then "section 480 of the Companies Act 2006", then "Limited Liability Partnership" OR "LLP"',
                    re.compile(r".*(Exempt|Exemption).*section 480 of the Companies Act 2006.*(Limited Liability Partnership|LLP).*"),
                ),
                (
                    '"Wedi\'i eithrio" or "Eithriad", then "adran 480 o Ddeddf Cwmnïau 2006"',
                    re.compile(r".*(Wedi'i eithrio|Eithriad).*adran 480 o Ddeddf Cwmnïau 2006.*(Partneriaeth Atebolrwydd Cyfyngedig|PAC).*"),
                ),
            ),
    },
    LP_SM_LP: {
        'StatementThatAccountsHaveBeenPreparedInAccordanceWithProvisionsSmallCompaniesRegime': (
            (
                '"Prepared in accordance with", then "provisions", then "small Limited Liability Partnership" OR "small LLP',
                re.compile(r".*Prepared in accordance with.*provisions.*(small Limited Liability Partnership|small LLP).*"),
            ),
            (
                '"Paratowyd yn unol â", then "y darpariaethau", then "Partneriaeth Atebolrwydd Cyfyngedig bach" or "PAC bach")',
                re.compile(r".*Paratowyd yn unol â.*y darpariaethau.*(Partneriaeth Atebolrwydd Cyfyngedig bach|PAC bach).*"),
            ),
        ),
    },
}

# Map of codes > dict of local name and warning level boolean
SINGLE_CONCEPT_EVALUATIONS: dict[str, dict[str, bool]] = {
    CO_MED_CO: {CONCEPT_MEDIUM_COMPANY: True},
    LP_MED_LP: {CONCEPT_MEDIUM_COMPANY: True},
    CO_QUAL_AUDIT: {CONCEPT_STATEMENT_PROVIDED: False},
    LP_QUAL_AUDIT: {CONCEPT_STATEMENT_PROVIDED: False},
}


class AccountStatus(Enum):
    AUDIT_EXEMPT_NO_REPORT = 'AuditExempt-NoAccountantsReport'
    AUDIT_EXEMPT_WITH_REPORT = 'AuditExemptWithAccountantsReport'


class NotTrading(Enum):
    CONCEPT_ENTITY_HAS_NEVER_TRADED = 'EntityHasNeverTraded'
    CONCEPT_ENTITY_NO_LONGER_TRADING = 'EntityNoLongerTradingButTradedInPast'


class ScopeAccounts(Enum):
    GROUP_ONLY = 'GroupAccountsOnly'
    CONSOLIDATED_GROUP = 'ConsolidatedGroupCompanyAccounts'


@dataclass
class CodeResult:
    success: bool = field(default=True)
    conceptLocalName: str | None = field(default=None)
    conceptList: list[str] | None = field(default=None)
    fact: ModelFact | None = field(default=None)
    message: str | None = field(default=None)
    warning: bool = field(default=False)


class HmrcLang(Enum):
    ENGLISH = 0
    WELSH = 1


@dataclass
class ValidateHmrc:
    modelXbrl: ModelXbrl
    _codeResultMap: dict[str, CodeResult] = field(default_factory=dict)

    def _checkValidFact(self, fact: ModelFact ) -> bool:
        if fact is not None:
            if not fact.isNil:
                return True
        return False

    def _errorOnMissingFact(self, conceptLocalName: str) -> None:
        """
        Logs an error explaining that a fact of the given concept was missing.
        :param conceptLocalName:
        """
        self.modelXbrl.error(
            CO_MISSING_ELEMENT,
            "Based on facts regarding Entity and Report Information provided, "
            "your document is expected to have a fact tagged with the following concept: %(conceptLocalName)s.",
            conceptLocalName=conceptLocalName
        )

    def _errorOnMissingFactText(self, code: str, result: CodeResult) -> None:
        """
        Logs an error on the `ModelXbrl` explaining the actual fact value did not match expected patterns.
        If a fact of the expected type did not exist, an additional error will be logged.
        :param code:
        :param result:
        """
        if result.fact is None and result.conceptLocalName is not None:
            self._errorOnMissingFact(result.conceptLocalName)
        self.modelXbrl.error(
            code,
            "The value for the the fact tagged with the concept %(conceptLocalName)s "
            "is missing the text for language '%(lang)s': %(message)s",
            conceptLocalName=result.conceptLocalName,
            lang=self._lang.name.title(),
            message=result.message,
            modelObject=result.fact,
        )

    def _setCode(self, code: str, result: CodeResult) -> CodeResult:
        """
        Caches the given result for the given code.
        :param code:
        :param result:
        :return: The given result.
        """
        self._codeResultMap[code] = result
        return result

    def _evaluateTextPattern(self, code: str) -> CodeResult:
        pattern: dict[str, tuple[tuple[str, Pattern[str]], ...]] | Any = TEXT_VALIDATION_PATTERNS.get(code, {})
        for conceptLocalName, textMatchers in pattern.items():
            facts = self._getFacts(conceptLocalName)
            if not facts:
                return CodeResult(
                    success=False,
                    conceptLocalName=conceptLocalName
                )
            for fact in facts:
                message, pattern = textMatchers[self._lang.value]
                match = pattern.match(fact.value, re.MULTILINE)
                if not match:
                    return CodeResult(
                        success=False,
                        conceptLocalName=conceptLocalName,
                        fact=fact,
                        message=message
                    )
        return CodeResult()

    def _evaluateDirectorFacts(self) -> CodeResult:
        """
        Logs an error when an audited report does not facts tagged with the concepts of "DateSigningDirectorsReport" or "DirectorSigningDirectorsReport"
        :return:
        """
        missingConcepts = []
        if not self._getAndCheckValidFacts(CONCEPT_DATE_SIGNING_DIRECTOR_REPORT):
            missingConcepts.append(CONCEPT_DATE_SIGNING_DIRECTOR_REPORT)
        if not self._getAndCheckValidFacts(CONCEPT_DIRECTOR_SIGNING_DIRECTORS_REPORT):
            missingConcepts.append(CONCEPT_DIRECTOR_SIGNING_DIRECTORS_REPORT)
        if len(missingConcepts) > 0:
            return CodeResult(
                    success=False,
                    conceptList=missingConcepts,
                    message="Facts tagged with the DateSigningDirectorsReport and DirectorSigningDirectorsReport must exist with non-nil values. "
                            "There are no facts tagged with the concepts: %(conceptList)s"
                )
        return CodeResult()

    def _evaluateAuditFacts(self) -> CodeResult:
        """
        Logs an error when an audited report does not facts tagged with the concepts of "DateAuditorsReport" and  "OpinionAuditorsOnEntity"
        as well as either "NameIndividualAuditor" or the combination of "NameSeniorStatutoryAuditor" and "NameEntityAuditors".
        :return:
        """
        missingConcepts = []
        if not self._getAndCheckValidFacts(CONCEPT_DATE_AUDITOR_REPORT):
            missingConcepts.append(CONCEPT_DATE_AUDITOR_REPORT)
        if not self._getAndCheckValidFacts(CONCEPT_OPINION_AUDITORS_ON_ENTITY):
            missingConcepts.append(CONCEPT_OPINION_AUDITORS_ON_ENTITY)
        if (not (self._getAndCheckValidFacts(CONCEPT_NAME_ENTITY_AUDITORS) and self._getAndCheckValidFacts(CONCEPT_NAME_SENIOR_STATUTORY_AUDITOR))
                and not self._getAndCheckValidFacts(CONCEPT_NAME_INDIVIDUAL_AUDITOR)):
            missingConcepts.append(CONCEPT_NAME_INDIVIDUAL_AUDITOR)
            missingConcepts.append(CONCEPT_NAME_SENIOR_STATUTORY_AUDITOR)
            missingConcepts.append(CONCEPT_NAME_ENTITY_AUDITORS)
        if len(missingConcepts) > 0:
            return CodeResult(
                success=False,
                conceptList=missingConcepts,
                message="An audited report must contain facts tagged with the concepts of DateAuditorsReport, OpinionAuditorsOnEntity "
                        "as well as either NameIndividualAuditor or the combination of NameSeniorStatutoryAuditor and NameEntityAuditors. "
                        "There are no facts tagged with the concepts: %(conceptList)s"
            )
        return CodeResult()

    def _evaluateRequiredSingleFact(self, code: str) -> CodeResult:
        """
        Logs a warning or error if the fact tied to the code is not found.
        """
        conceptAndLevel = SINGLE_CONCEPT_EVALUATIONS.get(code, {})
        for conceptLocalName, warning in conceptAndLevel.items():
            if not self._getAndCheckValidFacts(conceptLocalName):
                return CodeResult(
                    success=False,
                    warning=warning,
                    message="The concept of %(conceptLocalName)s must exist and have a non-nil value.",
                    conceptLocalName=conceptLocalName,
                )
        return CodeResult()

    def _evaluateProfLossFact(self) -> CodeResult:
        """
        Logs an error company or LLP does not facts tagged with the concept of ProfitLoss or
        EntityTradingStatus with the dimension of EntityTradingStatusDimension/(EntityHasNeverTraded OR EntityNoLongerTradingButTradedInPast).
        :param prefix:
        """
        trading = False
        for fact in self._getFacts(CONCEPT_ENTITY_TRADING_STATUS):
            if fact is None or fact.context is None:
                continue
            for qname, value in fact.context.qnameDims.items():
                if qname.localName == CONCEPT_ENTITY_TRADING_STATUS_DIMENSION:
                    if value.xValue.localName in {
                        NotTrading.CONCEPT_ENTITY_NO_LONGER_TRADING.value,
                        NotTrading.CONCEPT_ENTITY_HAS_NEVER_TRADED.value,
                    }:
                        trading = True
        if not self._getAndCheckValidFacts(CONCEPT_PROFIT_LOSS) and not trading:
            return CodeResult(
                    success=False,
                    message="A fact tagged with ProfitLoss must exist if a fact tagged with EntityTradingStatus"
                            "with the dimension of EntityTradingStatusDimension/(EntityHasNeverTraded OR EntityNoLongerTradingButTradedInPast)"
                            "does not exist or has a nil value'"
                )
        return CodeResult()

    def _evaluateCode(self, code: str) -> CodeResult:
        """
        Evaluates whether the conditions associated with the given code pass.
        Result is cached.
        :param code:
        :return: Result object that can be used for additional logic or to trigger an error.
        """
        if code in self._codeResultMap:
            return self._codeResultMap[code]
        if code in TEXT_VALIDATION_PATTERNS:
            result = self._evaluateTextPattern(code)
        elif code in (CO_AUDIT, LP_AUDIT):
            result = self._evaluateAuditFacts()
        elif code == CO_DIR_REP:
            result = self._evaluateDirectorFacts()
        elif code in SINGLE_CONCEPT_EVALUATIONS:
            result = self._evaluateRequiredSingleFact(code)
        elif code in (CO_PROF_LOSS, LP_PROF_LOSS):
            result = self._evaluateProfLossFact()
        return self._setCode(code, result)

    def _getFacts(self, conceptLocalName: str) -> list[ModelFact]:
        return [f for f in self.modelXbrl.factsByLocalName.get(conceptLocalName, set()) if f is not None]

    def _getAndCheckValidFacts(self, conceptLocalName: str) -> bool:
        facts = self._getFacts(conceptLocalName)
        if any(self._checkValidFact(x) for x in facts):
            return True
        return False

    @cached_property
    def _lang(self) -> HmrcLang:
        """
        Determines if the language is set to Welsh, otherwise defaults to English.
        """
        for fact in self._getFacts(CONCEPT_REPORT_PRINCIPAL_LANGUAGE):
            if fact is None or fact.context is None:
                continue
            for qname, value in fact.context.qnameDims.items():
                if qname.localName == CONCEPT_LANGUAGES_DIMENSION:
                    if value.xValue.localName == CONCEPT_WELSH:
                        return HmrcLang.WELSH
        return HmrcLang.ENGLISH

    def _yieldErrorOrWarning(self, code: str, result: CodeResult) -> None:
        """
        Logs an error on the `ModelXbrl` explaining the actual fact value did not match expected patterns.
        If a fact of the expected type did not exist, an additional error will be logged.
        :param code:
        :param result:
        """
        if result.message is None:
            result.message = ''
        if not result.warning:
            self.modelXbrl.error(
                code,
                msg=result.message,
                conceptList=result.conceptList,
                conceptLocalName=result.conceptLocalName,
                message=result.message,
                modelObject=result.fact,
            )
        else:
            self.modelXbrl.warning(
                code,
                msg=result.message,
                conceptList=result.conceptList,
                conceptLocalName=result.conceptLocalName,
                message=result.message,
                modelObject=result.fact,
            )

    @cached_property
    def accountStatus(self) -> str | None:
        facts = self._getFacts(CONCEPT_ACCOUNTS_STATUS)
        for fact in facts:
            if not self._checkValidFact(fact):
                continue
            for qname, value in fact.context.qnameDims.items():
                if qname.localName == CONCEPT_ACCOUNTS_STATUS_DIMENSION:
                    return cast(str, value.xValue.localName)
        return None

    @cached_property
    def accountsType(self) -> str | None:
        facts = self._getFacts(CONCEPT_ACCOUNTS_TYPE_FULL_OR_ABBREVIATED)
        for fact in facts:
            if not self._checkValidFact(fact):
                continue
            for qname, value in fact.context.qnameDims.items():
                if qname.localName == CONCEPT_ACCOUNTS_TYPE_DIMENSION:
                    return cast(str, value.xValue.localName)
        return None

    @cached_property
    def accountingStandardsApplied(self) -> str | None:
        facts = self._getFacts(CONCEPT_ACCOUNTING_STANDARDS_APPLIED)
        for fact in facts:
            if not self._checkValidFact(fact):
                continue
            for qname, value in fact.context.qnameDims.items():
                if qname.localName == CONCEPT_ACCOUNTING_STANDARDS_DIMENSION:
                    return cast(str, value.xValue.localName)
        return None

    @cached_property
    def applicableLegislation(self) -> str | None:
        facts = self._getFacts(CONCEPT_APPLICABLE_LEGISLATION)
        for fact in facts:
            if not self._checkValidFact(fact):
                continue
            for qname, value in fact.context.qnameDims.items():
                if qname.localName == CONCEPT_APPLICABLE_LEGISLATION_DIMENSION:
                    return cast(str, value.xValue.localName)
        return None

    @cached_property
    def isEntityDormant(self) -> bool:
        facts = self._getFacts(CONCEPT_ENTITY_DORMANT)
        if not facts:
            return False
        return all(not f.isNil and isinstance(f.xValue, bool) and f.xValue for f in facts)

    @cached_property
    def legalFormEntity(self) -> str | None:
        facts = self._getFacts(CONCEPT_LEGAL_FORM_ENTIY)
        for fact in facts:
            if not self._checkValidFact(fact):
                continue
            for qname, value in fact.context.qnameDims.items():
                if qname.localName == CONCEPT_LEGAL_FORM_ENTIY_DIMENSION:
                    return cast(str, value.xValue.localName)
        return None

    @cached_property
    def scopeAccounts(self) -> str | None:
        facts = self._getFacts(CONCEPT_SCOPE_ACCOUNTS)
        for fact in facts:
            if not self._checkValidFact(fact):
                continue
            for qname, value in fact.context.qnameDims.items():
                if qname.localName == CONCEPT_SCOPE_ACCOUNTS_DIMENSION:
                    return cast(str, value.xValue.localName)
        return None

    def validate(self) -> None:
        """
        Find the appropriate set of validations to run on this document and runs them.
        """
        if self.accountStatus in {
            AccountStatus.AUDIT_EXEMPT_NO_REPORT.value,
            AccountStatus.AUDIT_EXEMPT_WITH_REPORT.value,
        }:
            if self.isEntityDormant:
                if self.legalFormEntity == CONCEPT_LLP:
                    self.validateUnauditedDormantLLP()
                else:
                    self.validateUnauditedDormantCompany()
            elif self.accountingStandardsApplied == CONCEPT_MICRO_ENTITIES:
                if self.legalFormEntity == CONCEPT_LLP:
                    self.validateUnauditedMicroLLP()
                else:
                    self.validateUnauditedMicroCompany()
            elif self.accountsType == CONCEPT_ABRIDGED_ACCOUNTS:
                if self.legalFormEntity == CONCEPT_LLP:
                    self.validateUnauditedLLPAbridgedAccounts()
                else:
                    self.validateUnauditedCompanyAbridgedAccounts()
            elif self.accountsType == CONCEPT_ABBREVIATED_ACCOUNTS:
                if self.legalFormEntity == CONCEPT_LLP:
                    self.validateUnauditedLLPAbbreviatedAccounts()
                else:
                    self.validateUnauditedCompanyAbbreviatedAccounts()
            elif self.scopeAccounts in {
                ScopeAccounts.GROUP_ONLY.value,
                ScopeAccounts.CONSOLIDATED_GROUP.value,
            }:
                if self.legalFormEntity == CONCEPT_LLP:
                    self.validateUnauditedLLPGroupAccounts()
                else:
                    self.validateUnauditedCompanyGroupAccounts()
            elif (self.applicableLegislation == CONCEPT_SMALL_COMPANY_REGIME_FOR_ACCOUNTS
                  and not (self.scopeAccounts in {
                        ScopeAccounts.GROUP_ONLY.value,
                        ScopeAccounts.CONSOLIDATED_GROUP.value,
                    } and self.accountsType == CONCEPT_ABRIDGED_ACCOUNTS)):
                if self.legalFormEntity == CONCEPT_LLP:
                    self.validateUnauditedLLPFullAccounts()
                else:
                    self.validateUnauditedSmallCompanyFullAccounts()
        elif self.accountStatus == CONCEPT_AUDITED:
            if self.accountsType == CONCEPT_ABRIDGED_ACCOUNTS:
                if self.legalFormEntity == CONCEPT_LLP:
                    self.validateAuditedAbridgedLLPAccounts()
                else:
                    self.validateAuditedCompanyAbridgedAccounts()
            elif self.applicableLegislation == CONCEPT_SMALL_COMPANY_REGIME_FOR_ACCOUNTS:
                if self.legalFormEntity == CONCEPT_LLP:
                    self.validateAuditedSmallLLP()
                else:
                    self.validateAuditedSmallCompany()
            elif self.applicableLegislation == CONCEPT_MEDIUM_COMPANIES_REGIME_FOR_ACCOUNTS:
                if self.legalFormEntity == CONCEPT_LLP:
                    self.validateAuditedMediumLLP()
                else:
                    self.validateAuditedMediumCompany()
            elif self.accountingStandardsApplied == CONCEPT_MICRO_ENTITIES:
                if self.legalFormEntity == CONCEPT_LLP:
                    self.validateAuditedMicroLLP()
                else:
                    self.validateAuditedMicroCompany()
            elif not (self.scopeAccounts in {
                ScopeAccounts.GROUP_ONLY.value,
                ScopeAccounts.CONSOLIDATED_GROUP.value,
            } or self.accountsType == CONCEPT_ABRIDGED_ACCOUNTS or self.applicableLegislation == CONCEPT_SMALL_COMPANY_REGIME_FOR_ACCOUNTS):
                if self.legalFormEntity == CONCEPT_LLP:
                    self.validateAuditedOtherLLP()
                else:
                    self.validateAuditedOtherCompany()

    def validateAuditedAbridgedLLPAccounts(self) -> None:
        """
        Checks conditions applicable to audited abridged LLP accounts:
        Lp.Abrid, lp.Audit, and Lp.Smlp
        """
        result = self._evaluateCode(LP_ABRID)
        if not result.success:
            self._errorOnMissingFactText(LP_ABRID, result)
        result = self._evaluateCode(LP_SM_LP)
        if not result.success:
            self._errorOnMissingFactText(LP_SM_LP, result)
        result = self._evaluateCode(LP_AUDIT)
        if not result.success:
            self._yieldErrorOrWarning(LP_AUDIT, result)

    def validateAuditedCompanyAbridgedAccounts(self) -> None:
        """
        Checks conditions applicable to audited company abridged accounts:
        Co.Abrid, Co.Audit, and Co.SmCo
        """
        result = self._evaluateCode(CO_ABRID)
        if not result.success:
            self._errorOnMissingFactText(CO_ABRID, result)
        result = self._evaluateCode(CO_SM_CO)
        if not result.success:
            self._errorOnMissingFactText(CO_SM_CO, result)
        result = self._evaluateCode(CO_AUDIT)
        if not result.success:
            self._yieldErrorOrWarning(CO_AUDIT, result)

    def validateAuditedMediumCompany(self) -> None:
        """
        Checks conditions applicable to audited medium company filings:
        Co.Audit, and Co.MedCo
        """
        result = self._evaluateCode(CO_MED_CO)
        if not result.success:
            self._yieldErrorOrWarning(CO_MED_CO, result)
        result = self._evaluateCode(CO_AUDIT)
        if not result.success:
            self._yieldErrorOrWarning(CO_AUDIT, result)

    def validateAuditedMediumLLP(self) -> None:
        """
        Checks conditions applicable to audited medium llp filings:
        Lp.Audit, and Lp.MedLp
        """
        result = self._evaluateCode(LP_MED_LP)
        if not result.success:
            self._yieldErrorOrWarning(LP_MED_LP, result)
        result = self._evaluateCode(LP_AUDIT)
        if not result.success:
            self._yieldErrorOrWarning(LP_AUDIT, result)

    def validateAuditedMicroCompany(self) -> None:
        """
        Checks conditions applicable to audited micro company filings:
        Co.Micro and (Co.SmCo or Co.ProfLoss)
        """
        result = self._evaluateCode(CO_MICRO)
        if not result.success:
            self._errorOnMissingFactText(CO_MICRO, result)
        result = self._evaluateCode(CO_SM_CO)
        if not result.success:
            pLResult = self._evaluateCode(CO_PROF_LOSS)
            if not pLResult.success:
                self._errorOnMissingFactText(CO_SM_CO, result)
                self._yieldErrorOrWarning(CO_PROF_LOSS, pLResult)

    def validateAuditedMicroLLP(self) -> None:
        """
        Checks conditions applicable to audited micro LLP filings:
        LP.Micro and (Lp.SmLp or Lp.ProfLoss)
        """
        result = self._evaluateCode(LP_MICRO)
        if not result.success:
            self._errorOnMissingFactText(LP_MICRO, result)
        result = self._evaluateCode(LP_SM_LP)
        if not result.success:
            pLResult = self._evaluateCode(LP_PROF_LOSS)
            if not pLResult.success:
                self._errorOnMissingFactText(LP_SM_LP, result)
                self._yieldErrorOrWarning(LP_PROF_LOSS, pLResult)

    def validateAuditedOtherCompany(self) -> None:
        """
        Checks conditions applicable to audited other company filings:
        Co.Audit, Co.DirReport, Co.QualAudit, and Co.ProfLoss
        """
        result = self._evaluateCode(CO_AUDIT)
        if not result.success:
            self._errorOnMissingFactText(CO_AUDIT, result)
        result = self._evaluateCode(CO_DIR_REP)
        if not result.success:
            self._yieldErrorOrWarning(CO_DIR_REP, result)
        result = self._evaluateCode(CO_QUAL_AUDIT)
        if not result.success:
            self._yieldErrorOrWarning(CO_QUAL_AUDIT, result)
        result = self._evaluateCode(CO_PROF_LOSS)
        if not result.success:
            self._yieldErrorOrWarning(CO_PROF_LOSS, result)

    def validateAuditedOtherLLP(self) -> None:
        """
        Checks conditions applicable to audited other LLP filings:
        Lp.Audit, Lp.QualAudit and Lp.ProfLoss
        """
        result = self._evaluateCode(LP_AUDIT)
        if not result.success:
            self._errorOnMissingFactText(LP_AUDIT, result)
        result = self._evaluateCode(LP_QUAL_AUDIT)
        if not result.success:
            self._yieldErrorOrWarning(LP_QUAL_AUDIT, result)
        result = self._evaluateCode(LP_PROF_LOSS)
        if not result.success:
            self._yieldErrorOrWarning(LP_PROF_LOSS, result)

    def validateAuditedSmallCompany(self) -> None:
        """
        Checks conditions applicable to audited small company filings:
        Co.Audit and Co.SmCo
        """
        result = self._evaluateCode(CO_SM_CO)
        if not result.success:
            self._errorOnMissingFactText(CO_SM_CO, result)
        result = self._evaluateCode(CO_AUDIT)
        if not result.success:
            self._yieldErrorOrWarning(CO_AUDIT, result)

    def validateAuditedSmallLLP(self) -> None:
        """
        Checks conditions applicable to audited small LLP filings:
        Lp.Audit and Lp.Smlp
        """
        result = self._evaluateCode(LP_SM_LP)
        if not result.success:
            self._errorOnMissingFactText(LP_SM_LP, result)
        result = self._evaluateCode(LP_AUDIT)
        if not result.success:
            self._yieldErrorOrWarning(LP_AUDIT, result)

    def validateUnauditedCompanyAbbreviatedAccounts(self) -> None:
        """
        Checks conditions applicable to unaudited company abbreviated accounts:
        Co.Sec777, Co.AuditNR, Co.DirResp, and Co.SmCo
        """
        result = self._evaluateCode(CO_SEC_477)
        if not result.success:
            self._errorOnMissingFactText(CO_SEC_477, result)
        result = self._evaluateCode(CO_AUDIT_NR)
        if not result.success:
            self._errorOnMissingFactText(CO_AUDIT_NR, result)
        result = self._evaluateCode(CO_DIR_RESP)
        if not result.success:
            self._errorOnMissingFactText(CO_DIR_RESP, result)
        result = self._evaluateCode(CO_SM_CO)
        if not result.success:
            self._errorOnMissingFactText(CO_SM_CO, result)

    def validateUnauditedCompanyAbridgedAccounts(self) -> None:
        """
        Checks conditions applicable to unaudited company abridged accounts:
        Co.Sec777, Co.AuditNR, Co.DirResp, Co.SmCo, Co.Abrid
        """
        result = self._evaluateCode(CO_SEC_477)
        if not result.success:
            self._errorOnMissingFactText(CO_SEC_477, result)
        result = self._evaluateCode(CO_AUDIT_NR)
        if not result.success:
            self._errorOnMissingFactText(CO_AUDIT_NR, result)
        result = self._evaluateCode(CO_DIR_RESP)
        if not result.success:
            self._errorOnMissingFactText(CO_DIR_RESP, result)
        result = self._evaluateCode(CO_SM_CO)
        if not result.success:
            self._errorOnMissingFactText(CO_SM_CO, result)
        result = self._evaluateCode(CO_ABRID)
        if not result.success:
            self._errorOnMissingFactText(CO_ABRID, result)

    def validateUnauditedCompanyGroupAccounts(self) -> None:
        """
        Checks conditions applicable to unaudited company abridged accounts:
        Co.Sec477, Co.AuditNR, Co.DirResp, and Co.SmCo
        """
        result = self._evaluateCode(CO_SEC_477)
        if not result.success:
            self._errorOnMissingFactText(CO_SEC_477, result)
        result = self._evaluateCode(CO_AUDIT_NR)
        if not result.success:
            self._errorOnMissingFactText(CO_AUDIT_NR, result)
        result = self._evaluateCode(CO_DIR_RESP)
        if not result.success:
            self._errorOnMissingFactText(CO_DIR_RESP, result)
        result = self._evaluateCode(CO_SM_CO)
        if not result.success:
            self._errorOnMissingFactText(CO_SM_CO, result)

    def validateUnauditedDormantCompany(self) -> None:
            """
            Checks conditions applicable to unaudited dormant companies:
            Co.Sec480 and Co.AuditNR and Co.DirResp and (Co.Micro or Co.SmCo).
            """
            result = self._evaluateCode(CO_SEC_480)
            if not result.success:
                self._errorOnMissingFactText(CO_SEC_480, result)
            result = self._evaluateCode(CO_AUDIT_NR)
            if not result.success:
                self._errorOnMissingFactText(CO_AUDIT_NR, result)
            result = self._evaluateCode(CO_DIR_RESP)
            if not result.success:
                self._errorOnMissingFactText(CO_DIR_RESP, result)

            result = self._evaluateCode(CO_MICRO)
            if not result.success:
                smCoResult = self._evaluateCode(CO_SM_CO)
                if not smCoResult.success:
                    self._errorOnMissingFactText(CO_MICRO, result)
                    self._errorOnMissingFactText(CO_SM_CO, smCoResult)

    def validateUnauditedDormantLLP(self) -> None:
        """
        Checks conditions applicable to unaudited dormant LLPs:
        LP.MemResp and LP.Sec480 and LP.SmLP).
        """
        result = self._evaluateCode(LP_MEM_RESP)
        if not result.success:
            self._errorOnMissingFactText(LP_MEM_RESP, result)
        result = self._evaluateCode(LP_SEC_480)
        if not result.success:
            self._errorOnMissingFactText(LP_SEC_480, result)
        result = self._evaluateCode(LP_SM_LP)
        if not result.success:
            self._errorOnMissingFactText(LP_SM_LP, result)

    def validateUnauditedLLPAbbreviatedAccounts(self) -> None:
        """
        Checks conditions applicable to unaudited LLP abbreviated accounts:
        Lp.Sec777, LP.MemResp, and Lp.SmLp
        """
        result = self._evaluateCode(LP_SEC_477)
        if not result.success:
            self._errorOnMissingFactText(LP_SEC_477, result)
        result = self._evaluateCode(LP_MEM_RESP)
        if not result.success:
            self._errorOnMissingFactText(LP_MEM_RESP, result)
        result = self._evaluateCode(LP_SM_LP)
        if not result.success:
            self._errorOnMissingFactText(LP_SM_LP, result)

    def validateUnauditedLLPAbridgedAccounts(self) -> None:
        """
        Checks conditions applicable to unaudited LLP abridged accounts:
        Lp.Sec777, LP.MemResp, Lp.SmLp, and Lp.Abrid
        """
        result = self._evaluateCode(LP_SEC_477)
        if not result.success:
            self._errorOnMissingFactText(LP_SEC_477, result)
        result = self._evaluateCode(LP_MEM_RESP)
        if not result.success:
            self._errorOnMissingFactText(LP_MEM_RESP, result)
        result = self._evaluateCode(LP_SM_LP)
        if not result.success:
            self._errorOnMissingFactText(LP_SM_LP, result)
        result = self._evaluateCode(LP_ABRID)
        if not result.success:
            self._errorOnMissingFactText(LP_ABRID, result)

    def validateUnauditedLLPFullAccounts(self) -> None:
        """
        Checks conditions applicable to unaudited LLP full accounts:
        Lp.Sec777, LP.MemResp, and Lp.SmLp
        """
        result = self._evaluateCode(LP_SEC_477)
        if not result.success:
            self._errorOnMissingFactText(LP_SEC_477, result)
        result = self._evaluateCode(LP_MEM_RESP)
        if not result.success:
            self._errorOnMissingFactText(LP_MEM_RESP, result)
        result = self._evaluateCode(LP_SM_LP)
        if not result.success:
            self._errorOnMissingFactText(LP_SM_LP, result)

    def validateUnauditedLLPGroupAccounts(self) -> None:
        """
        Checks conditions applicable to unaudited LLP group accounts:
        Lp.Sec777, LP.MemResp, and Lp.SmLp
        """
        result = self._evaluateCode(LP_SEC_477)
        if not result.success:
            self._errorOnMissingFactText(LP_SEC_477, result)
        result = self._evaluateCode(LP_MEM_RESP)
        if not result.success:
            self._errorOnMissingFactText(LP_MEM_RESP, result)
        result = self._evaluateCode(LP_SM_LP)
        if not result.success:
            self._errorOnMissingFactText(LP_SM_LP, result)

    def validateUnauditedMicroCompany(self) -> None:
        """
        Checks conditions applicable to unaudited micro companies:
        Co.Sec477 and Co.AuditNR and Co.DirResp and Co.Micro.
        """
        result = self._evaluateCode(CO_SEC_477)
        if not result.success:
            self._errorOnMissingFactText(CO_SEC_477, result)
        result = self._evaluateCode(CO_AUDIT_NR)
        if not result.success:
            self._errorOnMissingFactText(CO_AUDIT_NR, result)
        result = self._evaluateCode(CO_DIR_RESP)
        if not result.success:
            self._errorOnMissingFactText(CO_DIR_RESP, result)
        result = self._evaluateCode(CO_MICRO)
        if not result.success:
            self._errorOnMissingFactText(CO_MICRO, result)

    def validateUnauditedMicroLLP(self) -> None:
        """
        Checks conditions applicable to unaudited micro companies:
        Lp.Sec477 and Lp.MemResp and Lp.Micro.
        """
        result = self._evaluateCode(LP_SEC_477)
        if not result.success:
            self._errorOnMissingFactText(LP_SEC_477, result)
        result = self._evaluateCode(LP_MEM_RESP)
        if not result.success:
            self._errorOnMissingFactText(LP_MEM_RESP, result)
        result = self._evaluateCode(LP_MICRO)
        if not result.success:
            self._errorOnMissingFactText(LP_MICRO, result)

    def validateUnauditedSmallCompanyFullAccounts(self) -> None:
        """
        Checks conditions applicable to unaudited small company full accounts:
        Co.Sec777, Co.AuditNR, Co.DirResp, and Co.SmCo
        """
        result = self._evaluateCode(CO_SEC_477)
        if not result.success:
            self._errorOnMissingFactText(CO_SEC_477, result)
        result = self._evaluateCode(CO_AUDIT_NR)
        if not result.success:
            self._errorOnMissingFactText(CO_AUDIT_NR, result)
        result = self._evaluateCode(CO_DIR_RESP)
        if not result.success:
            self._errorOnMissingFactText(CO_DIR_RESP, result)
        result = self._evaluateCode(CO_SM_CO)
        if not result.success:
            self._errorOnMissingFactText(CO_SM_CO, result)



from __future__ import annotations

import multiprocessing
import os.path
import tempfile
import zipfile
from collections import defaultdict
from collections.abc import Generator
from contextlib import ExitStack
from contextlib import nullcontext
from dataclasses import dataclass
from heapq import heapreplace
from pathlib import PurePath, PurePosixPath
from typing import Any, Callable, ContextManager, TYPE_CHECKING, cast
from unittest.mock import patch

from lxml import etree

from arelle.WebCache import WebCache
from tests.integration_tests.integration_test_util import get_test_data
from tests.integration_tests.validation.conformance_suite_config import ConformanceSuiteConfig

if TYPE_CHECKING:
    from _pytest.mark import ParameterSet


original_normalize_url_function = WebCache.normalizeUrl


def normalize_url_function(config: ConformanceSuiteConfig) -> Callable[[WebCache, str, str | None], str]:
    def normalize_url(self: WebCache, url: str, base: str | None = None) -> str:
        assert config.url_replace is not None
        if url.startswith(config.url_replace):
            return url.replace(config.url_replace, f'{config.prefixed_local_filepath}/')
        return cast(str, original_normalize_url_function(self, url, base))
    return normalize_url


def get_test_data_mp_wrapper(args_kws: tuple[list[Any], dict[str, Any]]) -> list[ParameterSet]:
    args, kws = args_kws
    return get_test_data(args, **kws)


def get_test_shards(config: ConformanceSuiteConfig) -> list[tuple[list[str], frozenset[str]]]:
    path_strs: list[str] = []
    if zipfile.is_zipfile(config.prefixed_local_filepath):
        with zipfile.ZipFile(config.prefixed_local_filepath) as zip_file:
            _collect_zip_test_cases(zip_file, config.file, path_strs)
    else:
        _collect_dir_test_cases(config.prefixed_local_filepath, config.file, path_strs)
    path_strs = sorted(path_strs)
    assert path_strs

    @dataclass(frozen=True)
    class Path:
        path: str
        plugins: tuple[str, ...]
        runtime: float
    paths_by_plugins: dict[tuple[str, ...], list[Path]] = defaultdict(list)
    for path_str in path_strs:
        path_plugins: set[str] = set()
        for prefix, additional_plugins in config.additional_plugins_by_prefix:
            if path_str.startswith(prefix):
                path_plugins.update(additional_plugins)
        paths_by_plugins[tuple(path_plugins)].append(
            Path(path=path_str, plugins=tuple(path_plugins), runtime=config.approximate_relative_timing.get(path_str, 1)))
    paths_in_runtime_order: list[Path] = sorted((path for paths in paths_by_plugins.values() for path in paths),
        key=lambda path: path.runtime, reverse=True)
    runtime_by_plugins: dict[tuple[str, ...], float] = {plugins: sum(path.runtime for path in paths)
        for plugins, paths in paths_by_plugins.items()}
    total_runtime = sum(runtime_by_plugins.values())
    shards_by_plugins: dict[tuple[str, ...], list[tuple[float, list[str]]]] = {}
    remaining_shards = config.shards
    for i, (plugins, _) in enumerate(paths_by_plugins.items()):
        n_shards = (remaining_shards
            if i == len(paths_by_plugins) - 1
            else 1 + round(runtime_by_plugins[plugins] / total_runtime * (config.shards - len(paths_by_plugins))))
        remaining_shards -= n_shards
        shards_by_plugins[plugins] = [(0, []) for _ in range(n_shards)]
    assert remaining_shards == 0
    for path in paths_in_runtime_order:
        shards_for_plugins = shards_by_plugins[path.plugins]
        shard_runtime, shard = shards_for_plugins[0]
        shard.append(path.path)
        heapreplace(shards_for_plugins, (shard_runtime + path.runtime, shard))
    assert shards_by_plugins.keys() == {()} | {tuple(plugins) for _, plugins in config.additional_plugins_by_prefix}
    # Sort shards by runtime so CI nodes are more likely to pick shards with similar runtimes.
    time_ordered_shards = sorted(
        (runtime, plugin_group, paths)
        for plugin_group, runtime_paths in shards_by_plugins.items()
        for runtime, paths in runtime_paths
    )
    shards = [
        (paths, frozenset(plugin_group))
        for _, plugin_group, paths in time_ordered_shards
    ]
    assert sorted(path for paths, _ in shards for path in paths) == path_strs
    return shards


def _collect_zip_test_cases(zip_file: zipfile.ZipFile, file_path: str, path_strs: list[str]) -> None:
    zip_files = zip_file.namelist()
    if file_path not in zip_files:
        # case insensitive search (necessary for EFM suite).
        matching_files = [
            zf for zf in zip_files
            if zf.casefold() == file_path.casefold()
        ]
        if len(matching_files) != 1:
            raise RuntimeError(f"Unable to find referenced test case file {file_path}.")
        file_path = matching_files[0]

    with zip_file.open(file_path) as fh:
        tree = etree.parse(fh)
    for test_case_index in _collect_test_case_paths(file_path, tree, path_strs):
        _collect_zip_test_cases(zip_file, test_case_index, path_strs)


def _collect_dir_test_cases(file_path_prefix: str, file_path: str, path_strs: list[str]) -> None:
    full_file_path = os.path.join(file_path_prefix, file_path)
    tree = etree.parse(full_file_path)
    for test_case_index in _collect_test_case_paths(file_path, tree, path_strs):
        _collect_dir_test_cases(file_path_prefix, test_case_index, path_strs)


def _collect_test_case_paths(file_path: str, tree: etree._ElementTree, path_strs: list[str]) -> Generator[str, None, None]:
    testcases_element = _get_elem_by_local_name(tree, 'testcases')
    if testcases_element is not None:
        test_root = testcases_element.get('root', '')
        # replace backslashes with forward slashes, e.g. in
        # 616-definition-syntax/616-14-RXP-definition-link-validations\616-14-RXP-definition-link-validations-testcase.xml
        for elem in testcases_element.findall('{*}testcase'):
            yield str(PurePosixPath(file_path).parent / test_root / cast(str, elem.get('uri')).replace('\\', '/'))
    else:
        assert _get_elem_by_local_name(tree, 'testcase') is not None, f'unexpected file is neither test case nor index of test cases {file_path}'
        path_strs.append(file_path)


def _get_elem_by_local_name(tree: etree._ElementTree, local_name: str) -> etree._Element | None:
    return tree.getroot() if tree.getroot().tag.split('}')[-1] == local_name else tree.find(f'{{*}}{local_name}')


def get_conformance_suite_arguments(config: ConformanceSuiteConfig, filename: str,
        additional_plugins: frozenset[str], build_cache: bool, offline: bool, log_to_file: bool,
        expected_failure_ids: frozenset[str], expected_empty_testcases: frozenset[str],
        shard: int | None) -> tuple[list[Any], dict[str, Any]]:
    use_shards = shard is not None
    optional_plugins = set()
    if build_cache:
        optional_plugins.add('CacheBuilder')
    plugins = config.plugins | additional_plugins | optional_plugins
    args = [
        '--file', filename,
        '--keepOpen',
        '--validate',
    ]
    if plugins:
        args.extend(['--plugins', '|'.join(sorted(plugins))])
    shard_str = f'-s{shard}' if use_shards else ''
    if build_cache:
        args.extend(['--cache-builder-path', f'conf-{config.name}{shard_str}-cache.zip'])
    if config.capture_warnings:
        args.append('--testcaseResultsCaptureWarnings')
    if log_to_file:
        args.extend([
            '--csvTestReport', f'conf-{config.name}{shard_str}-report.csv',
            '--logFile', f'conf-{config.name}{shard_str}-log.txt',
        ])
    if offline or not config.network_or_cache_required:
        args.extend(['--internetConnectivity', 'offline'])
    kws = dict(
        expected_failure_ids=expected_failure_ids,
        expected_empty_testcases=expected_empty_testcases,
        expected_model_errors=config.expected_model_errors,
        required_locale_by_ids=config.required_locale_by_ids,
        strict_testcase_index=config.strict_testcase_index,
    )
    return args + config.args, kws


def get_conformance_suite_test_results(
        config: ConformanceSuiteConfig,
        shards: list[int],
        build_cache: bool = False,
        log_to_file: bool = False,
        offline: bool = False) -> list[ParameterSet]:
    assert len(shards) == 0 or config.shards != 1, \
        'Conformance suite configuration must specify shards if --shard is passed'
    if shards:
        return get_conformance_suite_test_results_with_shards(
            config=config, shards=shards, build_cache=build_cache, log_to_file=log_to_file, offline=offline
        )
    else:
        return get_conformance_suite_test_results_without_shards(
            config=config, build_cache=build_cache, log_to_file=log_to_file, offline=offline
        )


def get_conformance_suite_test_results_with_shards(  # type: ignore[return]
        config: ConformanceSuiteConfig,
        shards: list[int],
        build_cache: bool = False,
        log_to_file: bool = False,
        offline: bool = False) -> list[ParameterSet]:
    tempfiles: list[Any] = []
    try:
        with ExitStack() as exit_stack:
            # delete=False and subsequent .close and unlink are for Windows compatibility.  bpo-14243
            tempfiles.extend(exit_stack.enter_context(tempfile.NamedTemporaryFile(dir='.', mode='wb', suffix='.xml', delete=False)) for _ in shards)
            tasks = []
            for shard, testcase_file in zip(shards, tempfiles):
                test_shards = get_test_shards(config)
                test_paths, additional_plugins = test_shards[shard]
                zip_path = config.prefixed_local_filepath
                all_test_paths = {path for test_paths, _ in test_shards for path in test_paths}
                unrecognized_expected_empty_testcases = config.expected_empty_testcases - all_test_paths
                assert not unrecognized_expected_empty_testcases, f'Unrecognized expected empty testcases: {unrecognized_expected_empty_testcases}'
                expected_empty_testcases = config.expected_empty_testcases.intersection(test_paths)
                unrecognized_expected_failure_ids = {id.rsplit(':', 1)[0] for id in config.expected_failure_ids} - all_test_paths
                assert not unrecognized_expected_failure_ids, f'Unrecognized expected failure IDs: {unrecognized_expected_failure_ids}'
                expected_failure_ids = frozenset(id for id in config.expected_failure_ids if id.rsplit(':', 1)[0] in test_paths)

                root = etree.Element('testcases')
                tree = etree.ElementTree(root)
                pathlib_zip_path = PurePath(zip_path)
                for test_path in test_paths:
                    etree.SubElement(root, 'testcase', uri=str((pathlib_zip_path / test_path).as_posix()))
                tree.write(testcase_file, encoding='utf-8', pretty_print=True, xml_declaration=True)
                testcase_file.flush()
                testcase_file.close()
                filename = testcase_file.name
                args = get_conformance_suite_arguments(
                    config=config, filename=filename, additional_plugins=additional_plugins,
                    build_cache=build_cache, offline=offline, log_to_file=log_to_file, shard=shard,
                    expected_failure_ids=expected_failure_ids, expected_empty_testcases=expected_empty_testcases,
                )
                tasks.append(args)
            url_context_manager: ContextManager[Any]
            if config.url_replace:
                url_context_manager = patch('arelle.WebCache.WebCache.normalizeUrl', normalize_url_function(config))
            else:
                url_context_manager = nullcontext()
            with url_context_manager, multiprocessing.Pool() as pool:
                results = pool.map(get_test_data_mp_wrapper, tasks)
                return [x for l in results for x in l]
    finally:
        for f in tempfiles:
            try:
                os.unlink(f.name)
            except OSError:
                pass


def get_conformance_suite_test_results_without_shards(
        config: ConformanceSuiteConfig,
        build_cache: bool = False,
        log_to_file: bool = False,
        offline: bool = False) -> list[ParameterSet]:
    additional_plugins = frozenset().union(*(plugins for _, plugins in config.additional_plugins_by_prefix))
    filename = os.path.join(config.prefixed_local_filepath, config.file)
    expected_empty_testcases = config.expected_empty_testcases
    expected_failure_ids = config.expected_failure_ids
    args, kws = get_conformance_suite_arguments(
        config=config, filename=filename, additional_plugins=additional_plugins,
        build_cache=build_cache, offline=offline, log_to_file=log_to_file, shard=None,
        expected_failure_ids=expected_failure_ids, expected_empty_testcases=expected_empty_testcases,
    )
    url_context_manager: ContextManager[Any]
    if config.url_replace:
        url_context_manager = patch('arelle.WebCache.WebCache.normalizeUrl', normalize_url_function(config))
    else:
        url_context_manager = nullcontext()
    with url_context_manager:
        return get_test_data(args, **kws)



from __future__ import annotations

import locale
import os.path
import re
from collections import Counter
from pathlib import PurePath
from typing import TYPE_CHECKING, cast

import pytest

from arelle import ModelDocument, PackageManager, PluginManager
from arelle.Cntlr import Cntlr
from arelle.CntlrCmdLine import parseAndRun
from arelle.FileSource import archiveFilenameParts

if TYPE_CHECKING:
    from _pytest.mark import ParameterSet


def get_document_id(doc: ModelDocument.ModelDocument) -> str:
    file_source = doc.modelXbrl.fileSource
    basepath = getattr(file_source, 'basefile', None)
    if basepath is None:
        # Try and find a basepath from referenced documents
        ref_file_source = next(iter(file_source.referencedFileSources.values()), None)
        if ref_file_source is not None:
            basepath = ref_file_source.basefile
    if basepath is None:
        # Try and find a basepath based on archive in path
        archivePathParts = archiveFilenameParts(doc.filepath)
        if archivePathParts is not None:
            return archivePathParts[1]
    if basepath is None:
        # Use file source URL as fallback if basepath not found
        basepath = os.path.dirname(file_source.url) + os.sep
    return PurePath(doc.filepath).relative_to(basepath).as_posix()


def get_test_data(
        args: list[str],
        expected_failure_ids: frozenset[str] = frozenset(),
        expected_empty_testcases: frozenset[str] = frozenset(),
        expected_model_errors: frozenset[str] = frozenset(),
        required_locale_by_ids: dict[str, re.Pattern[str]] | None = None,
        strict_testcase_index: bool = True,
) -> list[ParameterSet]:
    """
    Produces a list of Pytest Params that can be fed into a parameterized pytest function

    :param args: The args to be parsed by arelle in order to correctly produce the desired result set
    :param expected_failure_ids: The set of string test IDs that are expected to fail
    :param expected_empty_testcases: The set of paths of empty testcases, relative to the suite zip
    :param expected_model_errors: The set of error codes expected to be in the ModelXbrl errors
    :param required_locale_by_ids: The dict of IDs for tests which require a system locale matching a regex pattern.
    :param strict_testcase_index: Don't allow IOerrors when loading the testcase index
    :return: A list of PyTest Params that can be used to run a parameterized pytest function
    """
    if required_locale_by_ids is None:
        required_locale_by_ids = {}
    cntlr = parseAndRun(args)  # type: ignore[no-untyped-call]
    try:
        system_locale = locale.setlocale(locale.LC_CTYPE)
        results: list[ParameterSet] = []
        test_cases_with_no_variations = set()
        test_cases_with_unrecognized_type = {}
        model_document = cntlr.modelManager.modelXbrl.modelDocument
        test_cases: list[ModelDocument.ModelDocument] = []
        if strict_testcase_index and model_document.type == ModelDocument.Type.TESTCASESINDEX:
            model_errors = sorted(cntlr.modelManager.modelXbrl.errors)
            assert 'IOerror' not in model_errors, f'One or more testcases referenced by testcases index "{model_document.filepath}" were not found.'
        collect_test_data(
            cntlr=cntlr,
            expected_failure_ids=expected_failure_ids,
            expected_empty_testcases=expected_empty_testcases,
            expected_model_errors=expected_model_errors,
            required_locale_by_ids=required_locale_by_ids,
            system_locale=system_locale,
            results=results,
            model_document=model_document,
            test_cases=test_cases,
        )
        for test_case in sorted(test_cases, key=lambda doc: doc.uri):
            test_case_file_id = get_document_id(test_case)
            if test_case.type not in (ModelDocument.Type.TESTCASE, ModelDocument.Type.REGISTRYTESTCASE):
                test_cases_with_unrecognized_type[test_case_file_id] = test_case.type
            if not getattr(test_case, "testcaseVariations", None):
                test_cases_with_no_variations.add(test_case_file_id)
            else:
                for mv in test_case.testcaseVariations:
                    test_id = f'{test_case_file_id}:{mv.id}'
                    expected_failure = isExpectedFailure(test_id, expected_failure_ids, required_locale_by_ids, system_locale)
                    param = pytest.param(
                        {
                            'status': mv.status,
                            'expected': mv.expected,
                            'actual': mv.actual
                        },
                        id=test_id,
                        marks=[pytest.mark.xfail()] if expected_failure else [],
                    )
                    results.append(param)
        if test_cases_with_unrecognized_type:
            raise Exception(f"Some test cases have an unrecognized document type: {sorted(test_cases_with_unrecognized_type.items())}.")
        unrecognized_expected_empty_testcases = expected_empty_testcases.difference(map(get_document_id, test_cases))
        if unrecognized_expected_empty_testcases:
            raise Exception(f"Some expected empty test cases weren't found: {sorted(unrecognized_expected_empty_testcases)}.")
        unexpected_empty_testcases = test_cases_with_no_variations - expected_empty_testcases
        if unexpected_empty_testcases:
            raise Exception(f"Some test cases don't have any variations: {sorted(unexpected_empty_testcases)}.")
        test_id_frequencies = Counter(cast(str, p.id) for p in results)
        nonunique_test_ids = {test_id: count for test_id, count in test_id_frequencies.items() if count > 1}
        if nonunique_test_ids:
            raise Exception(f'Some test IDs are not unique.  Frequencies of nonunique test IDs: {nonunique_test_ids}.')
        nonexistent_expected_failure_ids = expected_failure_ids - test_id_frequencies.keys()
        if nonexistent_expected_failure_ids:
            raise Exception(f"Some expected failure IDs don't match any test cases: {sorted(nonexistent_expected_failure_ids)}.")
        nonexistent_required_locale_testcase_ids = required_locale_by_ids.keys() - test_id_frequencies.keys()
        if nonexistent_required_locale_testcase_ids:
            raise Exception(f"Some required locale IDs don't match any test cases: {sorted(nonexistent_required_locale_testcase_ids)}.")
        return results
    finally:
        cntlr.modelManager.close()
        PackageManager.close()  # type: ignore[no-untyped-call]
        PluginManager.close()  # type: ignore[no-untyped-call]


def collect_test_data(
        cntlr: Cntlr,
        expected_failure_ids: frozenset[str],
        expected_empty_testcases: frozenset[str],
        expected_model_errors: frozenset[str],
        required_locale_by_ids: dict[str, re.Pattern[str]],
        system_locale: str,
        results: list[ParameterSet],
        model_document: ModelDocument.ModelDocument,
        test_cases: list[ModelDocument.ModelDocument],
) -> None:
    if model_document.type == ModelDocument.Type.TESTCASESINDEX:
        for child_document in model_document.referencesDocument.keys():
            collect_test_data(
                cntlr=cntlr,
                expected_failure_ids=expected_failure_ids,
                expected_empty_testcases=expected_empty_testcases,
                expected_model_errors=expected_model_errors,
                required_locale_by_ids=required_locale_by_ids,
                system_locale=system_locale,
                results=results,
                model_document=child_document,
                test_cases=test_cases,
            )
    elif model_document.type == ModelDocument.Type.REGISTRY:
        test_cases.extend(model_document.referencesDocument.keys())
    elif model_document.type in (ModelDocument.Type.TESTCASE, ModelDocument.Type.REGISTRYTESTCASE):
        test_cases.append(model_document)
    elif model_document.type == ModelDocument.Type.INSTANCE:
        test_id = get_document_id(model_document)
        expected_failure = isExpectedFailure(test_id, expected_failure_ids, required_locale_by_ids, system_locale)
        model_errors = sorted(cntlr.modelManager.modelXbrl.errors)
        expected_model_errors_list = sorted(expected_model_errors)
        param = pytest.param(
            {
                'status': 'pass' if model_errors == expected_model_errors_list else 'fail',
                'expected': expected_model_errors_list,
                'actual': model_errors,
            },
            id=test_id,
            marks=[pytest.mark.xfail()] if expected_failure else [],
        )
        results.append(param)
    else:
        raise Exception('Unhandled model document type: {}'.format(model_document.type))


def isExpectedFailure(
        test_id: str,
        expected_failure_ids: frozenset[str],
        required_locale_by_ids: dict[str, re.Pattern[str]],
        system_locale: str,
) -> bool:
    if test_id in expected_failure_ids:
        return True
    if test_id in required_locale_by_ids:
        return not required_locale_by_ids[test_id].search(system_locale)
    return False



from pathlib import PurePath
from tests.integration_tests.validation.conformance_suite_config import ConformanceSuiteConfig

TIMING = {f'esef_conformance_suite_2022/tests/inline_xbrl/{k}': v for k, v in {
    'G2-1-2/index.xml': 9.040,
    'G2-1-3_1/index.xml': 4.550,
    'G2-1-3_2/index.xml': 4.311,
    'G2-2-1/index.xml': 4.426,
    'G2-2-2/index.xml': 4.538,
    'G2-2-3/index.xml': 13.277,
    'G2-2-6_1/index.xml': 8.715,
    'G2-2-6_2/index.xml': 6.868,
    'G2-3-1_1/index.xml': 4.284,
    'G2-3-1_2/index.xml': 10.577,
    'G2-4-1_1/index.xml': 8.515,
    'G2-4-1_2/index.xml': 4.372,
    'G2-4-1_3/index.xml': 4.351,
    'G2-4-2_1/index.xml': 4.360,
    'G2-4-2_2/index.xml': 4.204,
    'G2-5-1_1/index.xml': 8.611,
    'G2-5-1_2/index.xml': 8.516,
    'G2-5-1_3/index.xml': 6.493,
    'G2-5-2/index.xml': 8.554,
    'G2-5-3/index.xml': 4.339,
    'G2-5-4_1/index.xml': 4.180,
    'G2-5-4_2/index.xml': 4.316,
    'G2-5-4_3/index.xml': 4.210,
    'G2-6-1_1/index.xml': 6.488,
    'G2-6-1_2/index.xml': 6.349,
    'G2-6-2/index.xml': 4.367,
    'G2-7-1_1/index.xml': 4.256,
    'G2-7-1_2/index.xml': 4.269,
    'G3-1-1_1/index.xml': 14.848,
    'G3-1-1_2/index.xml': 9.528,
    'G3-1-2/index.xml': 15.612,
    'G3-1-5/index.xml': 17.297,
    'G3-2-2/index.xml': 4.195,
    'G3-2-3/index.xml': 4.277,
    'G3-4-2_1/index.xml': 4.350,
    'G3-4-2_2/index.xml': 4.394,
    'G3-4-2_3/index.xml': 4.348,
    'G3-4-2_4/index.xml': 6.442,
    'G3-4-3_1/index.xml': 6.497,
    'G3-4-3_2/index.xml': 6.362,
    'G3-4-4/index.xml': 4.223,
    'G3-4-5_1/index.xml': 4.211,
    'G3-4-5_2/index.xml': 6.315,
    'G3-4-6/index.xml': 6.322,
    'G3-4-7/index.xml': 4.301,
    'G3-5-1/index.xml': 4.296,
    'RTS_Annex_III_Par_1/index.xml': 6.585,
    'RTS_Annex_III_Par_3_G3-1-3/index.xml': 2.199,
    'RTS_Annex_II_Par_1/index.xml': 6.449,
    'RTS_Annex_II_Par_1_RTS_Annex_IV_par_7/index.xml': 4.415,
    'RTS_Annex_IV_Par_11_G3-2-2/index.xml': 6.419,
    'RTS_Annex_IV_Par_12_G2-2-4/index.xml': 15.164,
    'RTS_Annex_IV_Par_14_G2-5-1/index.xml': 4.323,
    'RTS_Annex_IV_Par_1_G2-1-4/index.xml': 4.293,
    'RTS_Annex_IV_Par_2_G2-1-1/index.xml': 6.409,
    'RTS_Annex_IV_Par_4_1/index.xml': 4.338,
    'RTS_Annex_IV_Par_4_2/index.xml': 4.300,
    'RTS_Annex_IV_Par_4_G1-1-1_G3-4-5/index.xml': 11.494,
    'RTS_Annex_IV_Par_5/index.xml': 6.410,
    'RTS_Annex_IV_Par_6/index.xml': 4.278,
    'RTS_Annex_IV_Par_8_G3-4-5/index.xml': 6.435,
    'RTS_Annex_IV_Par_9_Par_10_G1-4-1_G1-4-2_G3-3-1_G3-3-2/index.xml': 17.047,
    'RTS_Art_3/index.xml': 8.430,
    'RTS_Art_6_a/index.xml': 4.197,
}.items()}

config = ConformanceSuiteConfig(
    approximate_relative_timing=TIMING,
    args=[
        '--disclosureSystem', 'esef-2022',
        '--formula', 'run',
    ],
    expected_failure_ids=frozenset(f'esef_conformance_suite_2022/tests/{s}' for s in [
        # The following test cases fail because of the `tech_duplicated_facts1` formula which fires
        # incorrectly because it does not take into account the language attribute on the fact.
        # A fact can not be a duplicate fact if the language attributes are different.
        'inline_xbrl/RTS_Annex_IV_Par_12_G2-2-4/index.xml:TC5_valid'
    ]),
    file='esef_conformance_suite_2022/index_inline_xbrl.xml',
    info_url='https://www.esma.europa.eu/document/esef-conformance-suite-2022',
    local_filepath='esef_conformance_suite_2022.zip',
    name=PurePath(__file__).stem,
    plugins=frozenset({'validate/ESEF'}),
    public_download_url='https://www.esma.europa.eu/sites/default/files/library/esef_conformance_suite_2022.zip',
    shards=8,
)



from pathlib import PurePath
from tests.integration_tests.validation.conformance_suite_config import ConformanceSuiteConfig

TIMING = {
    'audited_abridged_LLP_accounts/testcase.xml': 15.487,
    'audited_company_abridged_accounts/testcase.xml': 22.816,
    'audited_medium_LLP/testcase.xml': 13.015,
    'audited_medium_company/testcase.xml': 12.880,
    'audited_small_LLP/testcase.xml': 11.441,
    'audited_small_company/testcase.xml': 11.741,
    'jfcvc/3312-testcase.xml': 8.009,
    'jfcvc/3315-testcase.xml': 7.891,
    'unaudited_company_abbreviated_accounts/testcase.xml': 23.917,
    'unaudited_company_abridged_accounts/testcase.xml': 26.535,
    'unaudited_company_group_accounts/testcase.xml': 25.746,
    'unaudited_dormant_company/testcase.xml': 30.359,
    'unaudited_dormant_llp/testcase.xml': 22.251,
    'unaudited_llp_abbreviated_accounts/testcase.xml': 19.648,
    'unaudited_llp_abridged_accounts/testcase.xml': 23.202,
    'unaudited_llp_full_accounts/testcase.xml': 19.965,
    'unaudited_llp_group_accounts/testcase.xml': 22.312,
    'unaudited_micro_company/testcase.xml': 30.898,
    'unaudited_micro_llp/testcase.xml': 22.313,
    'unaudited_small_company_full_accounts/testcase.xml': 22.876,
}

config = ConformanceSuiteConfig(
    approximate_relative_timing=TIMING,
    args=[
        '--hmrc',
    ],
    file='index.xml',
    info_url='https://www.gov.uk/government/organisations/hm-revenue-customs',
    local_filepath='HMRC',
    name=PurePath(__file__).stem,
    plugins=frozenset({'validate/HMRC'}),
    shards=4,
)



from pathlib import PurePath
from tests.integration_tests.validation.conformance_suite_config import ConformanceSuiteConfig

config = ConformanceSuiteConfig(
    file='calculation-1.1-conformance-2023-12-20/index.xml',
    info_url='https://specifications.xbrl.org/work-product-index-calculations-2-calculations-1-1.html',
    local_filepath='calculation-1.1-conformance-2023-12-20.zip',
    membership_url='https://www.xbrl.org/join',
    name=PurePath(__file__).stem,
    network_or_cache_required=False,
    plugins=frozenset({'loadFromOIM', '../../tests/plugin/testcaseCalc11ValidateSetup.py'}),
)



from pathlib import PurePath
from tests.integration_tests.validation.conformance_suite_config import ConformanceSuiteConfig

TIMING = {
    'br_kvk/2-04-testcase.xml': 6.145,
    'br_kvk/3-01-testcase.xml': 13.028,
    'br_kvk/4-07-testcase.xml': 6.638,
    'br_kvk/4-10-testcase.xml': 6.841,
    'br_kvk/4-12-testcase.xml': 6.916,
    'br_kvk/4-16-testcase.xml': 6.880,
    'br_kvk/4-17-testcase.xml': 0.724,
    'br_kvk/4-20-testcase.xml': 7.116,
    'fg_nl/04-testcase.xml': 0.011,
    'fg_nl/05-testcase.xml': 0.041,
    'fg_nl/09-testcase.xml': 0.043,
    'fg_nl/11-testcase.xml': 0.090,
    'fr_kvk/1-01-testcase.xml': 7.031,
    'fr_kvk/2-01-testcase.xml': 14.035,
    'fr_kvk/2-03-testcase.xml': 0.005,
    'fr_kvk/5-01-testcase.xml': 7.224,
    'fr_kvk/5-02-testcase.xml': 7.283,
    'fr_nl/1-01-testcase.xml': 0.017,
    'fr_nl/1-02-testcase.xml': 0.002,
    'fr_nl/1-03-testcase.xml': 0.001,
    'fr_nl/1-04-testcase.xml': 0.005,
    'fr_nl/1-05-testcase.xml': 0.001,
    'fr_nl/1-06-testcase.xml': 0.003,
    'fr_nl/2-03-testcase.xml': 0.003,
    'fr_nl/2-04-testcase.xml': 0.008,
    'fr_nl/2-05-testcase.xml': 0.106,
    'fr_nl/2-06-testcase.xml': 0.170,
    'fr_nl/2-07-testcase.xml': 1.993,
    'fr_nl/3-01-testcase.xml': 0.780,
    'fr_nl/3-02-testcase.xml': 0.867,
    'fr_nl/3-03-testcase.xml': 0.834,
    'fr_nl/3-04-testcase.xml': 2.372,
    'fr_nl/4-01-testcase.xml': 1.633,
    'fr_nl/4-02-testcase.xml': 0.845,
    'fr_nl/5-01-testcase.xml': 1.638,
    'fr_nl/5-03-testcase.xml': 0.806,
    'fr_nl/5-06-testcase.xml': 0.832,
    'fr_nl/5-11-testcase.xml': 0.031,
    'fr_nl/6-01-testcase.xml': 0.812,
}

config = ConformanceSuiteConfig(
    approximate_relative_timing=TIMING,
    args=[
        '--disclosureSystem', 'NT16-preview',
    ],
    file='index.xml',
    info_url='https://sbr-nl.nl/sites/default/files/bestanden/taxonomie/SBR%20Filing%20Rules%20NT16%20-%2020210301_0.pdf',
    local_filepath='nl_nt16',
    name=PurePath(__file__).stem,
    plugins=frozenset({'validate/NL'}),
    shards=4,
)
